# Form implementation generated from reading ui file 'project_item_frame_base.ui'
#
# Created by: PyQt6 UI code generator 6.4.2
#
# WARNING: Any manual changes made to this file will be lost when pyuic6 is
# run again.  Do not edit this file unless you know what you are doing.


from PySide6 import QtCore, QtGui, QtWidgets


class Ui_ProjectItemFrame(object):
  def setupUi(self, ProjectItemFrame):
    ProjectItemFrame.setObjectName("ProjectItemFrame")
    ProjectItemFrame.resize(1009, 50)
    sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Preferred, QtWidgets.QSizePolicy.Policy.Fixed)
    sizePolicy.setHorizontalStretch(0)
    sizePolicy.setVerticalStretch(0)
    sizePolicy.setHeightForWidth(ProjectItemFrame.sizePolicy().hasHeightForWidth())
    ProjectItemFrame.setSizePolicy(sizePolicy)
    ProjectItemFrame.setMaximumSize(QtCore.QSize(16777215, 50))
    ProjectItemFrame.setFrameShape(QtWidgets.QFrame.Shape.StyledPanel)
    ProjectItemFrame.setFrameShadow(QtWidgets.QFrame.Shadow.Raised)
    self.horizontalLayout_2 = QtWidgets.QHBoxLayout(ProjectItemFrame)
    self.horizontalLayout_2.setObjectName("horizontalLayout_2")
    self.horizontalLayout = QtWidgets.QHBoxLayout()
    self.horizontalLayout.setObjectName("horizontalLayout")
    self.projectCheckBox = QtWidgets.QCheckBox(parent=ProjectItemFrame)
    self.projectCheckBox.setMinimumSize(QtCore.QSize(0, 0))
    self.projectCheckBox.setMaximumSize(QtCore.QSize(50, 16777215))
    self.projectCheckBox.setLayoutDirection(QtCore.Qt.LayoutDirection.LeftToRight)
    self.projectCheckBox.setText("")
    self.projectCheckBox.setObjectName("projectCheckBox")
    self.horizontalLayout.addWidget(self.projectCheckBox)
    self.projectNameLabel = QtWidgets.QLabel(parent=ProjectItemFrame)
    self.projectNameLabel.setAlignment(QtCore.Qt.AlignmentFlag.AlignLeading|QtCore.Qt.AlignmentFlag.AlignLeft|QtCore.Qt.AlignmentFlag.AlignVCenter)
    self.projectNameLabel.setWordWrap(True)
    self.projectNameLabel.setObjectName("projectNameLabel")
    self.horizontalLayout.addWidget(self.projectNameLabel)
    self.modifiedDateTimeLabel = QtWidgets.QLabel(parent=ProjectItemFrame)
    self.modifiedDateTimeLabel.setAlignment(QtCore.Qt.AlignmentFlag.AlignLeading|QtCore.Qt.AlignmentFlag.AlignLeft|QtCore.Qt.AlignmentFlag.AlignVCenter)
    self.modifiedDateTimeLabel.setObjectName("modifiedDateTimeLabel")
    self.horizontalLayout.addWidget(self.modifiedDateTimeLabel)
    self.projectDocIdLabel = QtWidgets.QLabel(parent=ProjectItemFrame)
    self.projectDocIdLabel.setText("")
    self.projectDocIdLabel.setObjectName("projectDocIdLabel")
    self.horizontalLayout.addWidget(self.projectDocIdLabel)
    self.horizontalLayout_2.addLayout(self.horizontalLayout)

    self.retranslateUi(ProjectItemFrame)
    QtCore.QMetaObject.connectSlotsByName(ProjectItemFrame)

  def retranslateUi(self, ProjectItemFrame):
    _translate = QtCore.QCoreApplication.translate
    ProjectItemFrame.setWindowTitle(_translate("ProjectItemFrame", "Frame"))
    ProjectItemFrame.setToolTip(_translate("ProjectItemFrame", "PASTA project which can be uploaded to dataverse."))
    self.projectCheckBox.setToolTip(_translate("ProjectItemFrame", "Select the project before initiating the upload process."))
    self.projectNameLabel.setToolTip(_translate("ProjectItemFrame", "Name of the PASTA project to be uploaded."))
    self.projectNameLabel.setText(_translate("ProjectItemFrame", "Project Name"))
    self.modifiedDateTimeLabel.setToolTip(_translate("ProjectItemFrame", "Last modified time of the PASTA project."))
    self.modifiedDateTimeLabel.setText(_translate("ProjectItemFrame", "2024-01-16T10:52:15Z"))


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    ProjectItemFrame = QtWidgets.QFrame()
    ui = Ui_ProjectItemFrame()
    ui.setupUi(ProjectItemFrame)
    ProjectItemFrame.show()
    sys.exit(app.exec())



""" Represents QT dialog extension. """

#  PASTA-ELN and all its sub-parts are covered by the MIT license.
#
#  Copyright (c) 2024
#
#  Author: Jithu Murugan
#  Filename: dialog_extension.py
#
#  You should have received a copy of the license with this file. Please refer the license file for more information.
from PySide6 import QtCore
from PySide6.QtGui import QCloseEvent
from PySide6.QtWidgets import QDialog


class DialogExtension(QDialog):
  """
  Represents a dialog extension.

  Explanation:
      This class represents a dialog extension and provides a closeEvent method to handle the close event.
      It emits a closed signal when the dialog is closed.

  Args:
      None

  Returns:
      None
  """
  closed = QtCore.Signal()

  def closeEvent(self, close_event: QCloseEvent) -> None:
    """
    Overrides the close event method.

    Explanation:
        This method is called when the close event is triggered.
        It emits a closed signal.

    Args:
        close_event (QCloseEvent): The close event.
    """
    super().closeEvent(close_event)
    self.closed.emit()



""" Represents the controlled vocabulary frame. """
#  PASTA-ELN and all its sub-parts are covered by the MIT license.
#
#  Copyright (c) 2024
#
#  Author: Jithu Murugan
#  Filename: controlled_vocab_frame.py
#
#  You should have received a copy of the license with this file. Please refer the license file for more information.
import logging
from typing import Any

from PySide6.QtCore import QSize
from PySide6.QtWidgets import QComboBox, QFrame, QHBoxLayout, QPushButton, QSizePolicy

from pasta_eln.GUI.dataverse.primitive_compound_controlled_frame_base import Ui_PrimitiveCompoundControlledFrameBase
from pasta_eln.dataverse.utils import delete_layout_and_contents


class ControlledVocabFrame(Ui_PrimitiveCompoundControlledFrameBase):
  """
  Represents a controlled vocabulary frame.

  Explanation:
      This class represents a controlled vocabulary frame.
      It provides methods for adding new vocabulary entries, saving modifications, and handling button callbacks.

  """

  def __new__(cls, *_: Any, **__: Any) -> Any:
    """
    Creates and returns a new instance of the ControlledVocabFrame class.

    Returns:
        Any: A new instance of the ControlledVocabFrame class.

    Explanation:
        This method creates and returns a new instance of the ControlledVocabFrame class.
        It overrides the __new__ method of the superclass to ensure proper instantiation.
    """
    return super(ControlledVocabFrame, cls).__new__(cls)

  def __init__(self, meta_field: dict[str, Any]) -> None:
    """
    Initializes a new instance of the ControlledVocabFrame class.

    Args:
        meta_field (dict[str, Any]): The meta field data.

    Explanation:
        This method initializes a new instance of the ControlledVocabFrame class.
        It sets up the logger, creates a QFrame instance, and connects the add button callback.
        It then loads the UI based on the meta_field data.

    """
    self.logger = logging.getLogger(f"{__name__}.{self.__class__.__name__}")
    self.instance = QFrame()
    super().setupUi(self.instance)
    self.meta_field = meta_field
    self.addPushButton.clicked.connect(self.add_button_callback)
    self.load_ui()

  def load_ui(self) -> None:
    """
    Loads the UI for the controlled vocabulary frame.

    Explanation:
        This method loads the UI for the controlled vocabulary frame.
        It adds new vocabulary entries based on the meta_field values.
        If the meta_field has 'multiple' set to True, it adds multiple entries.
        If the meta_field has 'multiple' set to False, it adds a single entry.

    Args:
        self: The instance of the class.
    """
    self.logger.info("Loading controlled vocabulary frame ui..")
    value = self.meta_field.get('value')
    value_template = self.meta_field.get("valueTemplate")
    if self.meta_field.get('multiple'):
      if value:
        for value in value:
          self.add_new_vocab_entry(value_template, value)
      else:
        self.add_new_vocab_entry(value_template, None)
    else:
      self.add_new_vocab_entry(
        [value_template or ""],
        value)

  def add_button_callback(self) -> None:
    """
    Handles the button callback for adding a new vocabulary entry.

    Explanation:
        This method is called when the add button is clicked.
        It adds a new vocabulary entry based on the meta_field values.

    Args:
        self: The instance of the class.
    """
    self.logger.info("Adding new vocabulary entry, value: %s", self.meta_field)
    value_template = self.meta_field.get('valueTemplate')
    if self.meta_field.get('multiple'):
      self.add_new_vocab_entry(value_template, value_template[0]
      if value_template and len(value_template) > 0 else None)
    else:
      self.add_new_vocab_entry([value_template]
                               if value_template and len(value_template) > 0
                               else [], value_template)

  def add_new_vocab_entry(self,
                          controlled_vocabulary: list[str] | None = None,
                          value: str | None = None) -> None:
    """
    Adds a new vocabulary entry to the controlled vocabulary frame.

    Args:
        controlled_vocabulary (list[str] | None): The list of controlled vocabulary options.
        value (str | None): The initial value for the vocabulary entry.

    Explanation:
        This method adds a new vocabulary entry to the controlled vocabulary frame.
        It creates a new layout with a combo box and a delete button, and adds it to the main vertical layout.
        The combo box is populated with the controlled vocabulary options, and the initial value is set if provided.
        The delete button is connected to the delete_layout_and_contents function to handle deletion of the entry.
    """
    new_vocab_entry_layout = QHBoxLayout()
    new_vocab_entry_layout.setObjectName("vocabHorizontalLayout")
    combo_box = QComboBox(parent=self.instance)
    combo_box.setObjectName("vocabComboBox")
    combo_box.setToolTip("Select the controlled vocabulary.")
    combo_box.addItems(controlled_vocabulary or [])
    combo_box.setCurrentText(value or "")
    new_vocab_entry_layout.addWidget(combo_box)
    delete_push_button = QPushButton(parent=self.instance)
    delete_push_button.setText("Delete")
    delete_push_button.setToolTip("Delete this particular vocabulary entry.")
    size_policy = QSizePolicy(QSizePolicy.Policy.Fixed, QSizePolicy.Policy.Fixed)
    size_policy.setHorizontalStretch(0)
    size_policy.setVerticalStretch(0)
    size_policy.setHeightForWidth(delete_push_button.sizePolicy().hasHeightForWidth())
    delete_push_button.setSizePolicy(size_policy)
    delete_push_button.setMinimumSize(QSize(100, 0))
    delete_push_button.setObjectName("deletePushButton")
    new_vocab_entry_layout.addWidget(delete_push_button)
    delete_push_button.clicked.connect(lambda _: delete_layout_and_contents(new_vocab_entry_layout))
    self.mainVerticalLayout.addLayout(new_vocab_entry_layout)

  def save_modifications(self) -> None:
    """
    Saves the modifications made in the controlled vocabulary.

    Explanation:
        This method saves the modifications made in the controlled vocabulary.
        If the 'multiple' flag is True, it clears the 'value' list and retrieves the current text from each combo box.
        If the 'multiple' flag is False, it retrieves the current text from the single combo box.

    Args:
        self: The instance of the class.
    """
    value = self.meta_field.get('value')
    if self.meta_field['multiple'] and isinstance(value, list):
      value.clear()
      for layout_pos in reversed(range(self.mainVerticalLayout.count())):
        if vocab_horizontal_layout := self.mainVerticalLayout.itemAt(layout_pos).layout():
          combo_box = vocab_horizontal_layout.itemAt(0).widget()
          if text := combo_box.currentText():
            value.append(text)
      self.meta_field['value'] = list(set(value))
    elif layout := self.mainVerticalLayout.findChild(QHBoxLayout,
                                                     "vocabHorizontalLayout"):
      if combo_box := layout.itemAt(0).widget():
        self.meta_field['value'] = combo_box.currentText()
    else:
      self.logger.warning("Failed to save modifications, no layout found.")
    self.logger.info("Saved modifications successfully, value: %s", self.meta_field)



#!/usr/bin/python3
"""Commandline utility to admin the remote server"""
import sys, json, secrets, base64, os
from typing import Any
from datetime import datetime
from pathlib import Path
from zipfile import ZipFile, ZIP_DEFLATED
import keyring as cred
import requests
from requests.structures import CaseInsensitiveDict
from requests.auth import AuthBase
from PIL import Image, ImageDraw, ImageFont

def passwordEncrypt(message:str) -> bytes:
  """
  obfuscate message
  """
  return base64.b64encode(bytearray(message, encoding='utf-8'))
def passwordDecrypt(message:bytes) -> str:
  """
  de-obfuscate message
  """
  return base64.b64decode(message).decode('utf-8')

#global variables
headers:CaseInsensitiveDict[str]= CaseInsensitiveDict()
headers["Content-Type"] = "application/json"


def createUserDatabase(url:str, auth:AuthBase, userName:str) -> None:
  '''
  create a new user and database

  Args:
    url (string): url incl. http and port
    auth (object): HTTPBasicAuth object
    userName (string): user name, e.g. m.miller
  '''
  userPW = secrets.token_urlsafe(13)
  userDB = userName.replace('.','_')

  # create database
  resp = requests.put(f'{url}/{userDB}', headers=headers, auth=auth, timeout=10)
  if not resp.ok:
    print("**ERROR 1: put not successful",resp.reason)
    return

  # create user
  userDict = {"_id": f"org.couchdb.user:{userName}", "name": userName, "password": userPW, "roles": [f"{userDB}-W"], "type": "user", "orcid": ""}
  dataDict: dict[str, Any] = { "docs": [userDict] }
  data = json.dumps(dataDict)
  resp = requests.post(f'{url}/_users/_bulk_docs', headers=headers, auth=auth, data=data, timeout=10)
  if not resp.ok:
    print("**ERROR 2: post not successful",resp.reason)
    return

  # create _security in database
  dataDict = {
      "admins": {"names": [], "roles": [f"{userDB}-W"]},
      "members": {"names": [], "roles": [f"{userDB}-R"]},
  }
  data = json.dumps(dataDict)
  resp = requests.put(f'{url}/{userDB}/_security', headers=headers, auth=auth, data=data, timeout=10)
  if not resp.ok:
    print("**ERROR 3: post not successful",resp.reason)
    return

  # create _design/authentication in database
  dataDict = {"validate_doc_update": "function(newDoc, oldDoc, userCtx) {"+\
    "if (userCtx.roles.indexOf('"+userDB+"-W')!==-1){return;} "+\
    "else {throw({unauthorized:'Only Writers (W) may edit the database'});}}"}
  data = json.dumps(dataDict)
  resp = requests.put(f'{url}/{userDB}/_design/authentication', headers=headers, auth=auth, data=data, timeout=10)
  if not resp.ok:
    print("**ERROR 4: post not successful",resp.reason)
    return

  print('SUCCESS: Server interaction')
  #create image
  img = Image. new('RGB', (500, 500), color = (0, 65, 118))
  d = ImageDraw. Draw(img)
  font = ImageFont.truetype("arial.ttf", 24)
  d.text((30, 30),  "configuration name: remote", fill=(240,240,240), font=font)
  d.text((30, 70), f"user-name: {userName}", fill=(240,240,240), font=font)
  d.text((30,110), f"password: {userPW}",    fill=(240,240,240), font=font)
  d.text((30,150), f"database: {userDB}",    fill=(240,240,240), font=font)
  d.text((30,190),  "Remote configuration",  fill=(240,240,240), font=font)
  d.text((30,230), f"Server:   {url}",       fill=(240,240,240), font=font)
  img.save(f'{userDB}.png')
  #create key file
  dataDict = {"configuration name":"remote","user-name":userName,"password":userPW,"database":userDB,\
    "Remote configuration":"true","Server":url}
  dataBin = passwordEncrypt(json.dumps(dataDict))
  with open(f'{userDB}.key','bw') as fOut:
    fOut.write(dataBin)
  return


def listUsers(url:str, auth:AuthBase, verbose:bool=True) -> dict[str,Any]:
  '''
  list (and test) all users

  Args:
    url (string): url incl. http and port
    auth (object): HTTPBasicAuth object
    verbose (bool): verbose output

  Returns:
    dict: user information
  '''
  resp = requests.get(f'{url}/_users/_all_docs', headers=headers, auth=auth, timeout=10)
  if not resp.ok:
    print("**ERROR: get not successful",resp.reason)
    return {}
  if verbose:
    print("List of users:")
  results = {}
  for i in json.loads(resp.text)['rows']:
    if i['id']=='_design/_auth':
      continue
    if verbose:
      print(i['id'][17:]+'       key:'+i['key'][17:])
    responseI = requests.get(f'{url}/_users/'+i['id'], headers=headers, auth=auth, timeout=10)
    respI = json.loads(responseI.text)
    results[respI['name']] = respI['roles']
    if verbose:
      print('  Roles',respI['roles'])
      print('  Name ',respI['name'])
      if respI['name'].replace('.','_')+'-W' in respI['roles']:
        print('  -> corresponds to role-name convention')
      else:
        print('  -> DOES NOT correspond to role-name convention:',respI['name'])
      if i['id'].endswith(respI['name']):
        print('  -> corresponds to id-name convention')
      else:
        print('  -> DOES NOT correspond to id-name convention:',respI['name'])
      print('  Orcid',respI['orcid'])
  return {} if verbose else results


def listDB(url:str, auth:AuthBase, verbose:bool) -> dict[str,Any]:
  '''
  list (and test) all databases

  Args:
    url (string): url incl. http and port
    auth (object): HTTPBasicAuth object
    verbose (bool): verbose output

  Returns:
    dict: database information
  '''
  resp = requests.get(f'{url}/_all_dbs', headers=headers, auth=auth, timeout=10)
  if not resp.ok:
    print("**ERROR: get not successful",resp.reason)
    return {}
  if verbose:
    print("List of databases:")
  results = {}
  for i in json.loads(resp.text):
    if i in ['_replicator','_users']:
      continue
    if verbose:
      print(i)
    # test security
    responseI = requests.get(f'{url}/{i}/_security', headers=headers, auth=auth, timeout=10)
    respI = json.loads(responseI.text)
    security = [respI['admins']['roles'], respI['members']['roles']]
    if verbose:
      print('  Write',respI['admins']['roles'])
      print('  Read ',respI['members']['roles'])
      if (respI['admins']['roles'][0].endswith('-W') and
          respI['members']['roles'][0].endswith('-R') and
          respI['admins']['roles'][0].split('-')[0] == respI['members']['roles'][0].split('-')[0]):
        print('  -> everything ok')
      else:
        print('  -> **ERROR** security')
    # test authentication
    respI = requests.get(f'{url}/{i}/_design/authentication', headers=headers, auth=auth, timeout=10)
    respI = json.loads(respI.text)
    if 'validate_doc_update' in respI:
      respI = respI['validate_doc_update']
      respI = respI.split('indexOf')[1].split('!==')[0].strip()[2:-2]
      results[i] = security+[respI]
      if verbose:
        if respI == f'{i}-W':
          print('  -> everything ok')
        else:
          print('  -> **ERROR** authentication',respI)
    else:
      print("**ERROR <url>/<userDB>/_design/authentication' does not exist. Everybody can access!")
  return {} if verbose else results


def testUser(url:str, auth:AuthBase, userName:str, userPassword:str) -> None:
  '''
  test if configuration for this user is correct
  '''
  # Test if server exists
  resp = requests.get(url, headers=headers, timeout=10)
  if resp.ok:
    print("-> Server exists")
  else:
    print("**ERROR: Server cannot be reached")
    return
  authUser = requests.auth.HTTPBasicAuth(userName, userPassword)
  if auth is None:
    userDB = [userName.replace('.','_')]
  else:
    # test if configuration corresponds to naming convention
    users     = listUsers(url, auth, False)
    userDB    = [i[:-2] for i in users[userName]]  #list
  databases = listDB(url, auth, False)
  for iDB in userDB:
    if auth is not None:
      write = databases[iDB][0][0]
      read  = databases[iDB][1][0]
      authen= databases[iDB][2]
      if write.endswith('-W') and read.endswith('-R') and authen.endswith('-W') and \
        write[:-2] == read[:-2] and write[:-2] == authen[:-2]:
        print("-> User database and user-name correct")
      else:
        print("**ERROR: ",write,read,authen,iDB)
        break
    #test server with user credentials 1
    resp = requests.get(f'{url}/{iDB}', headers=headers, auth=authUser, timeout=10)
    if resp.ok:
      print("-> Database can be read")
    else:
      print("**ERROR: Database cannot be read")
      break
    #test server with user credentials 2
    resp = requests.get(f'{url}/{iDB}/_design/authentication', headers=headers, auth=authUser, timeout=10)
    if resp.ok:
      print("-> Authentication can be read")
    else:
      print("**ERROR: Authentication  cannot be read")
      break
  return


def testLocal(userName:str, password:str, database:str='') -> str:
  """
  test local server

  Args:
    userName (str): user name at local server
    password (str): password at local server
    database (str): couchdb database

  Returns:
    str: success and errors in '\n'-string
  """
  answer = ''
  resp = requests.get('http://127.0.0.1:5984', headers=headers, timeout=10)
  if resp.ok:
    answer += 'success: Local server exists\n'
  else:
    answer += 'ERROR: Local server is not working\n'
  authUser = requests.auth.HTTPBasicAuth(userName, password)
  resp = requests.get('http://127.0.0.1:5984/_all_dbs', headers=headers, auth=authUser, timeout=10)
  if resp.ok:
    answer += 'success: Local username and password ok\n'
  else:
    answer += 'ERROR: Local username or password incorrect\n'
  if database!='':
    resp = requests.get(f'http://127.0.0.1:5984/{database}', headers=headers, auth=authUser, timeout=10)
    if resp.ok:
      answer += 'success: Local database exists\n'
    else:
      answer += 'Error: Local database does not exist\n'
  return answer


def testRemote(url:str, userName:str, password:str, database:str) -> str:
  """
  test remote server

  Args:
    url (str): url of remote server
    userName (str): user name at remote server
    password (str): password at remote server
    database (str): couchdb database

  Returns:
    str: success and errors in '\n'-string
  """
  answer = ''
  resp = requests.get(url, headers=headers, timeout=10)
  if resp.ok:
    answer += 'success: Remote server exists\n'
  else:
    answer += 'ERROR: Remote server is not working\n'
  authUser = requests.auth.HTTPBasicAuth(userName, password)
  resp = requests.get(f'{url}/{database}', headers=headers, auth=authUser, timeout=10)
  if resp.ok:
    answer += 'success: Remote database exists\n'
  else:
    answer += 'ERROR: Remote username, password, database incorrect\n'
  return answer


def listDocuments(url:str, userName:str, password:str, database:str, full:bool=True) -> str:
  """
  list documents in database

  Args:
    url (str): url of remote server
    userName (str): user name at remote server
    password (str): password at remote server
    database (str): couchdb database
    full (bool): full list or just number

  Returns:
    str: success and errors in '\n'-string
  """
  authUser = requests.auth.HTTPBasicAuth(userName, password)
  resp = requests.get(f'{url}/{database}/_all_docs', headers=headers, auth=authUser, timeout=10)
  if resp.status_code != 200:
    print('**ERROR response for _all_dbs wrong', resp.text)
    print('Username and password', userName, password)
    return 'ERROR!'
  print(f"\nNumber of docs: {resp.json()['total_rows']}")
  if full:
    print(', '.join([doc['id'] for doc in resp.json()['rows']]))
  return 'Success'


def backupCouchDB(location:str='', userName:str='', password:str='') -> None:
  """
  Backup everything of the CouchDB installation across all databases and all configurations
  - remote location uses username/password combo in local keystore
  - local location requires username and password

  Args:
    location (str): 'local', 'remote', else: ask user via CLI
    userName (str): username
    password (str): password
  """
  # get username and password
  if not location:
    location = 'remote' if input('Enter location: [r] remote; else local: ')=='r' else 'local'
  if location=='local':
    location = '127.0.0.1'
    if not userName:
      userName = input('Enter local admin username: ').strip()
    if not password:
      password = input('Enter password: ').strip()
  elif location=='remote':
    try:
      myString = cred.get_password('pastaDB','admin')
      if myString is None:
        print("**ERROR Could not get credentials from keyring 2. Please create manually.")
        return
      location, userName, password = myString.split(':')
      print("URL and credentials successfully read from keyring")
    except Exception:
      print("**ERROR Could not get credentials from keyring 2b. Please create manually.")
      return
  else:
    print('**ERROR: wrong location given.')
    return
  # use information
  authUser = requests.auth.HTTPBasicAuth(userName, password)
  resp = requests.get(f'http://{location}:5984/_all_dbs', headers=headers, auth=authUser, timeout=10)
  if resp.status_code != 200:
    print('**ERROR response for _all_dbs wrong', resp.text)
    print('Username and password', userName, password)
    return
  timestamp = datetime.now().isoformat().split('.')[0].replace('-','').replace(':','')
  zipFileName = 'couchDB_backup_'+location.replace('.','')+'_'+timestamp
  print(f'Create zip-file {zipFileName}.zip')
  databases = resp.json()
  with ZipFile(f'{zipFileName}.zip', 'w', compression=ZIP_DEFLATED) as zipFile:
    for database in databases:
      if database.startswith('_'):
        print('Special database', database, ': Nothing to do')
      else:
        print('Backup normal database ',database)
        resp = requests.get(f'http://{location}:5984/{database}/_all_docs', headers=headers, auth=authUser, timeout=10)
        for item in resp.json()['rows']:
          docID = item['id']
          doc   = requests.get(f'http://{location}:5984/{database}/{docID}', headers=headers, auth=authUser, timeout=10).json()
          zipFile.writestr(f'{zipFileName}/research/{docID}', json.dumps(doc))
          if '_attachments' in doc:
            for att in doc['_attachments']:
              docAttach = requests.get(f'http://{location}:5984/{database}/{docID}/{att}',
                                        headers=headers, auth=authUser, timeout=10).json()
              zipFile.writestr(f'{zipFileName}/{database}/{docID}_attach/{att}', json.dumps(docAttach))
        #_design/authentication is automatically included
        #_security
        doc   = requests.get(f'http://{location}:5984/{database}/_security',
                              headers=headers, auth=authUser, timeout=10).json()
        zipFile.writestr(f'{zipFileName}/{database}/_security', json.dumps(doc))
    with open(Path.home()/'.pastaELN.json', encoding='utf-8') as fIn:
      configuration = json.loads(fIn.read())
      for projectG in configuration['projectGroups']:
        for site in ['local','remote']:
          subsection = configuration['projectGroups'][projectG][site]
          if 'cred' in subsection and ('user' not in subsection or 'password' not in subsection):
            key = cred.get_password('pastaDB', subsection['cred'])
            subsection['user'], subsection['password'] = ['',''] if key is None else key.split('bcA:Maw')
      zipFile.writestr(f'{zipFileName}/pastaELN.json', json.dumps(configuration))
  return


def restoreCouchDB(location:str='', userName:str='', password:str='', fileName:str='') -> None:
  """
  restore everything to the CouchDB installation across all databases and all configurations
  - remote location uses username/password combo in local keystore
  - local location requires username and password

  Args:
    location (str): 'local', 'remote', else: ask user via CLI
    userName (str): username
    password (str): password
    fileName (str): file used for restoration
  """
  # get username and password
  if not location:
    location = 'remote' if input('Enter location: [r]emote; else local: ')=='r' else 'local'
  if location=='local':
    location = '127.0.0.1'
    if not userName:
      userName = input('Enter username: ').strip()
    if not password:
      password = input('Enter password: ').strip()
  elif location=='remote':
    try:
      myString = cred.get_password('pastaDB','admin')
      if myString is None:
        print("**ERROR Could not get credentials from keyring 3. Please create manually.")
        return
      location, userName, password = myString.split(':')
      print("URL and credentials successfully read from keyring")
    except Exception:
      print("**ERROR Could not get credentials from keyring 3b. Please create manually.")
      return
  else:
    print('**ERROR: wrong location given.')
    return
  if not fileName:
    possFiles = [i for i in os.listdir('.') if i.startswith('couchDB') and i.endswith('.zip')]
    for idx, i in enumerate(possFiles):
      print(f'[{str(idx + 1)}] {i}')
    fileChoice = input(f'Which file to use for restored? (1-{len(possFiles)}) ')
    fileName = possFiles[int(fileChoice)-1] if fileChoice else possFiles[0]
  # use information
  authUser = requests.auth.HTTPBasicAuth(userName, password)
  with ZipFile(fileName, 'r', compression=ZIP_DEFLATED) as zipFile:
    files = [i for i in zipFile.namelist() if not i.endswith('/')]  #only files, no folders
    #first run through: create documents and design documents
    for fileI in files:
      fileParts = fileI.split('/')[1:]
      if fileParts==['pastaELN.json']: #do not recreate file, it is only there for manual recovery
        continue
      if len(fileParts)<2:
        print(f"**ERROR: Cannot process file {fileI}: does not have 1+2 parts")
        continue
      database = fileParts[0]
      docID = fileParts[1]
      if docID.endswith('_attach'):
        continue #Do in second loop
      #test if database is exists: create otherwise
      resp = requests.get(f'http://{location}:5984/{database}/_all_docs', headers=headers, auth=authUser, timeout=10)
      if resp.status_code != 200 and resp.json()['reason']=='Database does not exist.':
        resp = requests.put(f'http://{location}:5984/{database}', headers=headers, auth=authUser, timeout=10)
        if not resp.ok:
          print("**ERROR: could not create database",resp.reason)
          return
      #test if document is exists: create otherwise
      if docID=='_design':
        docID = '/'.join(fileParts[1:])
      resp = requests.get(f'http://{location}:5984/{database}/{docID}', headers=headers, auth=authUser, timeout=10)
      if resp.status_code != 200 and resp.json()['reason']=='missing':
        with zipFile.open(fileI) as dataIn:
          doc = json.loads( dataIn.read() )  #need doc conversion since deleted from it
          del doc['_rev']
          if '_attachments' in doc:
            del doc['_attachments']
          resp = requests.put(f'http://{location}:5984/{database}/{docID}', data=json.dumps(doc), headers=headers, auth=authUser, timeout=10)
          if not resp.ok:
            print("**ERROR: could not save document:",resp.reason, database, docID, '\n', doc)
            # don't print on success: reduce output
    #second run through: create attachments
    for fileI in files:
      fileParts = fileI.split('/')[1:]
      if fileParts==['pastaELN.json'] or fileParts[-1]=='' or fileParts[-2]=='_design': #do not recreate file, it is only there for manual recovery
        continue
      if len(fileParts)<2:
        print(f"**ERROR-Attachment: Cannot process file {fileI}: does not have 1+2 parts")
        continue
      database = fileParts[0]
      docID = fileParts[1]
      if not docID.endswith('_attach'):
        continue #Did already in the first loop
      #test if attachment exists: create otherwise
      attachPath = f'{docID[:-7]}/{fileParts[-1]}'
      resp = requests.get(f'http://{location}:5984/{database}/{attachPath}', headers=headers, auth=authUser, timeout=10)
      if resp.status_code == 404 and 'missing' in resp.json()['reason']:
        with zipFile.open(fileI) as dataIn:
          attachDoc = dataIn.read()
          resp = requests.get(f'http://{location}:5984/{database}/{docID[:-7]}', headers=headers, auth=authUser, timeout=10)
          headers['If-Match'] = resp.json()['_rev'] #will be overwritten each time
          resp = requests.put(f'http://{location}:5984/{database}/{attachPath}', data=attachDoc, headers=headers, auth=authUser, timeout=10)
          if not resp.ok:
            print('\n**ERROR: could not save attachment:',resp.reason, database, attachPath,'\n', doc)
  return


def main() -> None:
  '''
  Main function
  '''
  #set information once in keyring
  #  myString = url+':'+adminUserName+':'+adminPassword
  #  url without http and port
  #  cred.set_password('pastaDB','admin',myString)
  try:
    myString = cred.get_password('pastaDB','admin')
    if myString is None:
      print("**ERROR Could not get credentials from keyring 1. Please create manually.")
      print("import keyring as cred\nmyString = url+':'+adminUserName+':'+adminPassword\n#  url without http and port\ncred.set_password('pastaDB','admin',myString)")
      sys.exit(1)
    url, administrator, password = myString.split(':')
    print("URL and credentials successfully read from keyring")
  except Exception:
    print("Could not get credentials for the remote server from keyring.")
    ## URL
    url = input('Enter the URL without http and without port: ')
    if len(url)<2:
      print('* No legit URL entered: exit')
      sys.exit(1)
    ## User-name, password
    administrator = input('Enter the administrator username: ')
    password =      input('Enter the administrator password: ')
  #assemble information
  url = f'http://{url}:5984'
  auth = requests.auth.HTTPBasicAuth(administrator, password)

  print('\n-------------------------------------------------------------------------')
  print(  'Manage users and databases for PASTA-ELN on a remote couchDB installation')
  print(  '-------------------------------------------------------------------------')
  while True:
    print('\nCommands: [q]uit; [n]ew user; list [u]ser; list [d]atabases; '+\
          '[t]est username and password; [b]ackup data; [r]estore data; [c]redentials; [l]ist documents in database')
    command = input('> ')
    userName, userPassword = '', ''
    if command == 'q':
      break
    # ask questions for parameters
    if command in ['n', 't', 'l']:
      userName =      input('Enter the user-name, e.g. m.miller: ')
    if command in ['t', 'l']:
      userPassword =  input('Enter the user-password: ')
    # execute command
    if command == 'n' and userName and len(userName)>2:
      createUserDatabase(url, auth, userName)
    elif command == 'u':
      listUsers(url, auth)
    elif command == 'd':
      listDB(url, auth, True)
    elif command == 'b':
      backupCouchDB()
    elif command == 'r':
      restoreCouchDB()
    elif command == 'c':
      print('``` python')
      print('import json, requests')
      print('from requests.structures import CaseInsensitiveDict')
      print('headers:CaseInsensitiveDict[str]= CaseInsensitiveDict()')
      print('headers["Content-Type"] = "application/json"\n')
      print(f'url = "{url}"')
      print(f'administrator = "{administrator}"')
      print(f'password = "{password}"\n')
      print('auth = requests.auth.HTTPBasicAuth(administrator, password)')
      print('```')
    elif command == 'l':
      database = input('Enter the database: ')
      full     = input('Full output [yN] ')=='y'
      listDocuments(url, userName, userPassword, database, full)
    elif command == 't' and userName and userPassword and len(userName)>2 and len(userPassword)>2:
      testUser(url, auth, userName, userPassword)
    else:
      print("Unknown command or incomplete entries.")


if __name__ ==  '__main__':
  main()



""" Represents the possible upload status values. """
#  PASTA-ELN and all its sub-parts are covered by the MIT license.
#
#  Copyright (c) 2024
#
#  Author: Jithu Murugan
#  Filename: upload_status_values.py
#
#  You should have received a copy of the license with this file. Please refer the license file for more information.

from enum import Enum


class UploadStatusValues(Enum):
  """
  Represents the possible upload status values.

  Explanation:
      This enumeration class defines the possible values for the upload status.
      It provides a set of named constants to represent different upload statuses.

  Examples:
      UploadStatusValues.Queued
      UploadStatusValues.Uploading
      UploadStatusValues.Cancelled
      UploadStatusValues.Finished
      UploadStatusValues.Error
      UploadStatusValues.Warning
  """
  Queued = 1
  Uploading = 2
  Cancelled = 3
  Finished = 4
  Error = 5
  Warning = 6



""" widget that shows the details of the items """
import logging
from enum import Enum
from pathlib import Path
from typing import Any
from PySide6.QtWidgets import QScrollArea, QLabel, QTextEdit  # pylint: disable=no-name-in-module
from PySide6.QtCore import Qt, Slot # pylint: disable=no-name-in-module
from ..guiStyle import TextButton, Image, Label, showMessage, widgetAndLayout, getColor
from ._contextMenu import initContextMenu, executeContextMenu, CommandMenu
from ..fixedStringsJson import defaultDataHierarchyNode
from ..guiCommunicate import Communicate
from ..handleDictionaries import dict2ul
from ..miscTools import markdownStyler


class Details(QScrollArea):
  """ widget that shows the details of the items """
  def __init__(self, comm:Communicate):
    super().__init__()
    self.comm = comm
    comm.changeDetails.connect(self.change)
    comm.testExtractor.connect(self.testExtractor)
    self.doc:dict[str,Any]  = {}
    self.docID= ''
    self.rescaleTexts:list[QTextEdit] = []

    # GUI elements
    self.mainW, self.mainL = widgetAndLayout('V', None)
    self.setVerticalScrollBarPolicy(Qt.ScrollBarPolicy.ScrollBarAlwaysOn)
    self.setHorizontalScrollBarPolicy(Qt.ScrollBarPolicy.ScrollBarAlwaysOff)
    self.setWidgetResizable(True)
    self.setWidget(self.mainW)

    headerW, self.headerL = widgetAndLayout('H', self.mainL, top='s')
    headerW.setContextMenuPolicy(Qt.ContextMenuPolicy.CustomContextMenu)
    headerW.customContextMenuRequested.connect(lambda pos: initContextMenu(self, pos))
    self.specialW, self.specialL = widgetAndLayout('V', self.mainL, top='s')
    self.specialW.setContextMenuPolicy(Qt.ContextMenuPolicy.CustomContextMenu)
    self.specialW.customContextMenuRequested.connect(lambda pos: initContextMenu(self, pos))
    self.btnDetails = TextButton('Details', self, [Command.SHOW, 'Details'], self.mainL, \
                                 'Show / hide details', checkable=True, style='margin-top: 3px')
    self.metaDetailsW, self.metaDetailsL  = widgetAndLayout('V', self.mainL)
    self.metaDetailsW.setMaximumWidth(self.width())
    self.btnVendor = TextButton('Vendor metadata', self, [Command.SHOW, 'Vendor'], self.mainL, \
                                'Show / hide vendor metadata', checkable=True, style="margin-top: 15px")
    self.metaVendorW, self.metaVendorL = widgetAndLayout('V', self.mainL)
    self.metaVendorW.setMaximumWidth(self.width())
    self.btnUser = TextButton('User metadata', self, [Command.SHOW, 'User'], self.mainL, \
                              'Show / hide user metadata', checkable=True, style="margin-top: 15px")
    self.metaUserW, self.metaUserL     = widgetAndLayout('V', self.mainL)
    self.metaUserW.setMaximumWidth(self.width())
    self.btnDatabase = TextButton('Database details', self, [Command.SHOW,'Database'], self.mainL, \
                                  'Show / hide database details', checkable= True, style="margin-top: 15px")
    self.metaDatabaseW, self.metaDatabaseL = widgetAndLayout('V', self.mainL)
    self.metaDatabaseW.setMaximumWidth(self.width())
    self.mainL.addStretch(1)


  @Slot(str)
  def change(self, docID:str) -> None:
    """
    What happens when details should change

    Args:
      docID (str): document-id; '' string=draw nothing; 'redraw' implies redraw
    """
    logging.debug('details:changeDetails |%s|',docID)
    # Delete old widgets from layout
    for i in reversed(range(self.headerL.count())):
      self.headerL.itemAt(i).widget().setParent(None)
    for i in reversed(range(self.metaDetailsL.count())):
      self.metaDetailsL.itemAt(i).widget().setParent(None)
    if self.metaVendorL.itemAt(0) is not None:
      self.metaVendorL.itemAt(0).widget().setParent(None)
    if self.metaUserL.itemAt(0) is not None:
      self.metaUserL.itemAt(0).widget().setParent(None)
    for i in reversed(range(self.metaDatabaseL.count())):
      self.metaDatabaseL.itemAt(i).widget().setParent(None)
    for i in reversed(range(self.specialL.count())):
      self.specialL.itemAt(i).widget().setParent(None)
    self.specialW.hide()
    self.metaDetailsW.hide()
    self.metaVendorW.hide()
    self.metaUserW.hide()
    self.metaDatabaseW.hide()
    self.btnDetails.setChecked(True)
    self.btnVendor.setChecked(True)
    self.btnUser.setChecked(True)
    self.btnDatabase.setChecked(False)
    self.rescaleTexts = []
    if not docID:  #if given '' docID, return
      return
    # Create new
    if docID!='redraw':
      self.docID = docID
    if self.docID=='':
      return
    self.doc   = self.comm.backend.db.getDoc(self.docID)
    if '-name' not in self.doc:  #keep empty details and wait for user to click
      self.comm.changeTable.emit('','')
      return
    if self.doc['-type'][0]=='-' or self.doc['-type'][0] not in self.comm.backend.db.dataHierarchy:
      dataHierarchyNode = defaultDataHierarchyNode
    else:
      dataHierarchyNode = self.comm.backend.db.dataHierarchy[self.doc['-type'][0]]['meta']
    label = self.doc['-name'] if len(self.doc['-name'])<80 else self.doc['-name'][:77]+'...'
    Label(label,'h1', self.headerL)
    if 'metaVendor' not in self.doc:
      self.btnVendor.hide()
    if 'metUser' not in self.doc:
      self.btnUser.hide()
    for key in self.doc:
      size = self.comm.backend.configuration['GUI']['imageSizeDetails'] \
                        if hasattr(self.comm.backend, 'configuration') else 300
      if key=='image':
        Image(self.doc['image'], self.specialL, anyDimension=size)
        self.specialW.show()
      elif key=='content':
        text = QTextEdit()
        text.setMarkdown(self.doc['content'])
        text.setFixedHeight(int(size/3*2))
        text.setReadOnly(True)
        self.specialL.addWidget(text)
        self.specialW.show()
      elif key=='-tags':
        tags = ['_curated_' if i=='_curated' else f'#{i}' for i in self.doc[key]]
        tags = ['\u2605'*int(i[2]) if i[:2]=='#_' else i for i in tags]
        label = QLabel('Tags: '+' '.join(tags))
        label.setTextInteractionFlags(Qt.TextInteractionFlag.TextSelectableByMouse)
        self.metaDetailsL.addWidget(label)
      elif key[0] in ['_','-'] or key in ['shasum']:
        if key in ['_attachments']:
          continue
        label = QLabel(f'{key}: {str(self.doc[key])}')
        label.setWordWrap(True)
        label.setTextInteractionFlags(Qt.TextInteractionFlag.TextSelectableByMouse)
        self.metaDatabaseL.addWidget(label)
        self.btnDatabase.setChecked(False)
      elif key=='metaVendor':
        self.btnVendor.show()
        label = QLabel()
        label.setWordWrap(True)
        cssStyle = '<style> ul {list-style-type: none; padding-left: 0; margin: 0;} </style>'
        label.setText(cssStyle+dict2ul(self.doc[key]))
        label.setTextInteractionFlags(Qt.TextInteractionFlag.TextSelectableByMouse)
        self.metaVendorL.addWidget(label)
        self.metaVendorW.show()
      elif key=='metaUser':
        self.btnUser.show()
        label = QLabel()
        label.setWordWrap(True)
        cssStyle = '<style> ul {list-style-type: none; padding-left: 0; margin: 0;} </style>'
        label.setText(cssStyle+dict2ul(self.doc[key]))
        label.setTextInteractionFlags(Qt.TextInteractionFlag.TextSelectableByMouse)
        self.metaUserL.addWidget(label)
        self.metaUserW.show()
      else:
        link = False
        dataHierarchyItem = [i for group in dataHierarchyNode for i in dataHierarchyNode[group] if i['name']==key]
        if (isinstance(self.doc[key],str) and '\n' in self.doc[key]) or key=='comment':
          labelW, labelL = widgetAndLayout('H', self.metaDetailsL, top='s', bottom='s')
          labelL.addWidget(QLabel(f'{key}: '), alignment=Qt.AlignmentFlag.AlignTop)
          text = QTextEdit()
          text.setMarkdown(markdownStyler(self.doc[key]))
          bgColor = getColor(self.comm.backend, 'secondaryDark')
          fgColor = getColor(self.comm.backend, 'primaryText')
          text.setStyleSheet(f"QTextEdit {{ border: none; padding: 0px; background-color: {bgColor}; "\
                                f"color: {fgColor} }}")
          text.document().setTextWidth(labelW.width())
          self.rescaleTexts.append(text)
          height:int = text.document().size().toTuple()[1]    # type:ignore[index]
          text.setFixedHeight(height)
          text.setReadOnly(True)
          labelL.addWidget(text, stretch=1)
        else:
          if len(dataHierarchyItem)==1 and 'list' in dataHierarchyItem[0] and \
              not isinstance(dataHierarchyItem[0]['list'], list):                #choice among docType
            table  = self.comm.backend.db.getView('viewDocType/'+dataHierarchyItem[0]['list'])
            choices= [i for i in table if i['id']==self.doc[key]]
            if len(choices)==1:
              value = '\u260D '+choices[0]['value'][0]
              link = True
          elif isinstance(self.doc[key], list):
            value = ', '.join(self.doc[key])
          else:
            value = self.doc[key]
          label = Label(f'{key.capitalize()}: {value}', function=self.clickLink if link else None, docID=self.doc[key])
          label.setTextInteractionFlags(Qt.TextInteractionFlag.TextSelectableByMouse)
          self.metaDetailsL.addWidget(label)
        self.metaDetailsW.show()
    return


  def execute(self, command:list[Any]) -> None:
    """
    Hide / show the widget underneath the button

    Args:
      command (list): area to show/hide
    """
    if command[0] is Command.SHOW:
      if getattr(self, f'btn{command[1]}').isChecked(): #get button in question
        getattr(self, f'meta{command[1]}W').show()
      else:
        getattr(self, f'meta{command[1]}W').hide()
    elif isinstance(command[0], CommandMenu):
      if executeContextMenu(self, command):
        self.comm.changeTable.emit('','')
        self.comm.changeDetails.emit(self.doc['_id'])
    else:
      print("**ERROR details command unknown:",command)
    return


  @Slot()
  def testExtractor(self) -> None:
    """
    User selects to test extractor on this dataset
    """
    logging.debug('details:testExtractor')
    if len(self.doc)>1:
      path = Path(self.doc['-branch'][0]['path'])
      if not path.as_posix().startswith('http'):
        path = self.comm.backend.basePath/path
      report = self.comm.backend.testExtractor(path, outputStyle='html', recipe='/'.join(self.doc['-type']))
      showMessage(self, 'Report of extractor test', report, style='QLabel {min-width: 800px}')
    return


  def clickLink(self, label:str, docID:str) -> None:
    """
    Click link in details

    Args:
      label (str): label on link
      docID (str): docID to which to link
    """
    logging.debug('used link on %s|%s',label,docID)
    self.comm.changeDetails.emit(docID)
    return


  def resizeWidth(self, width:int) -> None:
    """ called if details is resized by splitter at the parent widget
    - resize all text-documents
    """
    if not self.rescaleTexts:
      return
    self.metaDetailsW.setFixedWidth(width)
    for text in self.rescaleTexts:
      text.document().setTextWidth(width)
      height:int = text.document().size().toTuple()[1] # type:ignore[index]
      text.setFixedHeight(height)
    return


class Command(Enum):
  """ Commands used in this file """
  SHOW             = 1



# Form implementation generated from reading ui file 'edit_metadata_summary_dialog_base.ui'
#
# Created by: PyQt6 UI code generator 6.4.2
#
# WARNING: Any manual changes made to this file will be lost when pyuic6 is
# run again.  Do not edit this file unless you know what you are doing.


from PySide6 import QtCore, QtGui, QtWidgets


class Ui_EditMetadataSummaryDialog(object):
  def setupUi(self, EditMetadataSummaryDialog):
    EditMetadataSummaryDialog.setObjectName("EditMetadataSummaryDialog")
    EditMetadataSummaryDialog.resize(1047, 486)
    self.gridLayout = QtWidgets.QGridLayout(EditMetadataSummaryDialog)
    self.gridLayout.setObjectName("gridLayout")
    self.buttonBox = QtWidgets.QDialogButtonBox(parent=EditMetadataSummaryDialog)
    self.buttonBox.setOrientation(QtCore.Qt.Orientation.Horizontal)
    self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.StandardButton.No|QtWidgets.QDialogButtonBox.StandardButton.Yes)
    self.buttonBox.setObjectName("buttonBox")
    self.gridLayout.addWidget(self.buttonBox, 1, 0, 1, 1)
    self.summaryTextEdit = QtWidgets.QTextEdit(parent=EditMetadataSummaryDialog)
    self.summaryTextEdit.setObjectName("summaryTextEdit")
    self.gridLayout.addWidget(self.summaryTextEdit, 0, 0, 1, 1)

    self.retranslateUi(EditMetadataSummaryDialog)
    self.buttonBox.accepted.connect(EditMetadataSummaryDialog.accept) # type: ignore
    self.buttonBox.rejected.connect(EditMetadataSummaryDialog.reject) # type: ignore
    QtCore.QMetaObject.connectSlotsByName(EditMetadataSummaryDialog)

  def retranslateUi(self, EditMetadataSummaryDialog):
    _translate = QtCore.QCoreApplication.translate
    EditMetadataSummaryDialog.setWindowTitle(_translate("EditMetadataSummaryDialog", "Metadata Summary"))


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    EditMetadataSummaryDialog = QtWidgets.QDialog()
    ui = Ui_EditMetadataSummaryDialog()
    ui.setupUi(EditMetadataSummaryDialog)
    EditMetadataSummaryDialog.show()
    sys.exit(app.exec())



# Form implementation generated from reading ui file 'create_type_dialog_base.ui'
#
# Created by: PyQt6 UI code generator 6.4.2
#
# WARNING: Any manual changes made to this file will be lost when pyuic6 is
# run again.  Do not edit this file unless you know what you are doing.


from PySide6 import QtCore, QtGui, QtWidgets


class Ui_CreateTypeDialogBase(object):
  def setupUi(self, CreateTypeDialogBase):
    CreateTypeDialogBase.setObjectName("CreateTypeDialogBase")
    CreateTypeDialogBase.resize(584, 375)
    self.gridLayout = QtWidgets.QGridLayout(CreateTypeDialogBase)
    self.gridLayout.setObjectName("gridLayout")
    self.mainVerticalLayout = QtWidgets.QVBoxLayout()
    self.mainVerticalLayout.setContentsMargins(20, -1, 20, -1)
    self.mainVerticalLayout.setObjectName("mainVerticalLayout")
    self.tileHorizontalLayout = QtWidgets.QHBoxLayout()
    self.tileHorizontalLayout.setObjectName("tileHorizontalLayout")
    self.titleLabel = QtWidgets.QLabel(parent=CreateTypeDialogBase)
    self.titleLabel.setMinimumSize(QtCore.QSize(120, 0))
    self.titleLabel.setObjectName("titleLabel")
    self.tileHorizontalLayout.addWidget(self.titleLabel)
    spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Minimum)
    self.tileHorizontalLayout.addItem(spacerItem)
    self.titleLineEdit = QtWidgets.QLineEdit(parent=CreateTypeDialogBase)
    self.titleLineEdit.setClearButtonEnabled(True)
    self.titleLineEdit.setObjectName("titleLineEdit")
    self.tileHorizontalLayout.addWidget(self.titleLineEdit)
    self.mainVerticalLayout.addLayout(self.tileHorizontalLayout)
    self.displayedTitleHorizontalLayout = QtWidgets.QHBoxLayout()
    self.displayedTitleHorizontalLayout.setObjectName("displayedTitleHorizontalLayout")
    self.typeLabel = QtWidgets.QLabel(parent=CreateTypeDialogBase)
    self.typeLabel.setMinimumSize(QtCore.QSize(120, 0))
    self.typeLabel.setObjectName("typeLabel")
    self.displayedTitleHorizontalLayout.addWidget(self.typeLabel)
    spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Minimum)
    self.displayedTitleHorizontalLayout.addItem(spacerItem1)
    self.displayedTitleLineEdit = QtWidgets.QLineEdit(parent=CreateTypeDialogBase)
    self.displayedTitleLineEdit.setClearButtonEnabled(True)
    self.displayedTitleLineEdit.setObjectName("displayedTitleLineEdit")
    self.displayedTitleHorizontalLayout.addWidget(self.displayedTitleLineEdit)
    self.mainVerticalLayout.addLayout(self.displayedTitleHorizontalLayout)
    spacerItem2 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Expanding)
    self.mainVerticalLayout.addItem(spacerItem2)
    self.checkBoxHorizontalLayout = QtWidgets.QHBoxLayout()
    self.checkBoxHorizontalLayout.setObjectName("checkBoxHorizontalLayout")
    self.structuralLevelCheckBox = QtWidgets.QCheckBox(parent=CreateTypeDialogBase)
    self.structuralLevelCheckBox.setObjectName("structuralLevelCheckBox")
    self.checkBoxHorizontalLayout.addWidget(self.structuralLevelCheckBox)
    self.mainVerticalLayout.addLayout(self.checkBoxHorizontalLayout)
    self.mainVerticalLayout.setStretch(0, 1)
    self.mainVerticalLayout.setStretch(1, 1)
    self.mainVerticalLayout.setStretch(2, 1)
    self.mainVerticalLayout.setStretch(3, 1)
    self.gridLayout.addLayout(self.mainVerticalLayout, 0, 0, 1, 1)
    self.buttonBox = QtWidgets.QDialogButtonBox(parent=CreateTypeDialogBase)
    self.buttonBox.setOrientation(QtCore.Qt.Orientation.Horizontal)
    self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.StandardButton.Cancel|QtWidgets.QDialogButtonBox.StandardButton.Ok)
    self.buttonBox.setObjectName("buttonBox")
    self.gridLayout.addWidget(self.buttonBox, 1, 0, 1, 1)

    self.retranslateUi(CreateTypeDialogBase)
    self.buttonBox.accepted.connect(CreateTypeDialogBase.accept) # type: ignore
    self.buttonBox.rejected.connect(CreateTypeDialogBase.reject) # type: ignore
    QtCore.QMetaObject.connectSlotsByName(CreateTypeDialogBase)

  def retranslateUi(self, CreateTypeDialogBase):
    _translate = QtCore.QCoreApplication.translate
    CreateTypeDialogBase.setWindowTitle(_translate("CreateTypeDialogBase", "Create a new data type"))
    self.titleLabel.setText(_translate("CreateTypeDialogBase", "Data type"))
    self.titleLineEdit.setToolTip(_translate("CreateTypeDialogBase", "Exclude titles which start with \'x\' (reserved for structure level titles) or whitespace"))
    self.titleLineEdit.setPlaceholderText(_translate("CreateTypeDialogBase", "Enter the data type"))
    self.typeLabel.setText(_translate("CreateTypeDialogBase", "Title"))
    self.displayedTitleLineEdit.setToolTip(_translate("CreateTypeDialogBase", "Enter displayed title for the new type, which can also be modified later in the main editor window"))
    self.displayedTitleLineEdit.setPlaceholderText(_translate("CreateTypeDialogBase", "Enter the displayed title"))
    self.structuralLevelCheckBox.setToolTip(_translate("CreateTypeDialogBase", "If this is a structural type, then title will be automatically populated as (x0, x1...xn). Next number will be chosen for xn from the existing list of structural items."))
    self.structuralLevelCheckBox.setText(_translate("CreateTypeDialogBase", "Is this a structural Type?"))


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    CreateTypeDialogBase = QtWidgets.QDialog()
    ui = Ui_CreateTypeDialogBase()
    ui.setupUi(CreateTypeDialogBase)
    CreateTypeDialogBase.show()
    sys.exit(app.exec())



""" TableViewModel Generic module used for the table views """
#  PASTA-ELN and all its sub-parts are covered by the MIT license.
#
#  Copyright (c) 2023
#
#  Author: Jithu Murugan
#  Filename: tableview_data_model.py
#
#  You should have received a copy of the license with this file. Please refer the license file for more information.

import logging
from typing import Any, Union

from PySide6.QtCore import QAbstractTableModel, QModelIndex, QPersistentModelIndex, Qt, Slot
from PySide6.QtWidgets import QWidget


class TableViewModel(QAbstractTableModel):
  """
  Abstracted data-model base for the data hierarchy table views
  """

  def __init__(self,
               parent: Union[QWidget | None] = None):
    """
    Initialize the data model representing attachments from data hierarchy document in the database
    Args:
      parent (QWidget): Parent view or widget
    """
    super().__init__(parent)
    self.logger = logging.getLogger(f"{__name__}.{self.__class__.__name__}")
    self.data_set: list[dict[Any, Any]] | Any = []
    self.data_name_map: dict[int, str] = {}
    self.header_values: list[str] = []
    self.columns_count = 0

  def hasChildren(self, parent: Union[QModelIndex, QPersistentModelIndex] = ...) -> bool:  # type: ignore[assignment]
    """
    Returns whether the model has children
    Args:
      parent (Union[QModelIndex, QPersistentModelIndex]): Parent index

    Returns: False since it's a tree view model

    """
    return False  # Since it's a table model, the children are not supported

  def headerData(self,
                 section: int,
                 orientation: Qt.Orientation,
                 role: int = Qt.ItemDataRole) -> Any:  # type: ignore[assignment]
    """
    Returns the header data from self.header_values
    Args:
      section (int): Index section of the table header
      orientation (Qt.Orientation): Orientation of the table header
      role (int): Display role for the table

    Returns (Any): The header name for the column represented by the section index

    """
    if orientation == Qt.Orientation.Horizontal and role == Qt.ItemDataRole.DisplayRole:
      return self.header_values[section].capitalize()
    return super().headerData(section, orientation, role)

  def update(self,
             updated_table_data: list[dict[Any, Any]] | Any) -> None:
    """
    Updating the table data model
    Args:
      updated_table_data (dict): Newly set data model

    Returns: None

    """
    self.logger.info("Table Data updated..")
    self.data_set = updated_table_data
    self.layoutChanged.emit()

  def rowCount(self,
               parent: Union[QModelIndex, QPersistentModelIndex] = ...) -> int:  # type: ignore[assignment]
    """
    Returns the row count for the table under the given parent
    Args:
      parent (Union[QModelIndex, QPersistentModelIndex]): Parent index

    Returns (int): Row count

    """
    return len(self.data_set) \
      if self.data_set \
      else 0

  def columnCount(self,
                  parent: Union[QModelIndex, QPersistentModelIndex] = ...) -> int:  # type: ignore[assignment]
    """
    Returns the column count for the table under the given parent
    Args:
      parent (Union[QModelIndex, QPersistentModelIndex]): Parent index

    Returns (int): Column count

    """
    return self.columns_count

  def setData(self,
              index: Union[QModelIndex, QPersistentModelIndex],
              value: Any,
              role: int = Qt.ItemDataRole) -> bool:  # type: ignore[assignment]
    """
    Sets the data for the table cell represented by the index. The data is set only for the below cases
     - EditRole: When the data in the cell is edited via line edit
     - UserRole: When the cell contains radio button and is set via required_column_delegate
    Args:
      index (Union[QModelIndex, QPersistentModelIndex]): Index with row & column representing the table cell
      value (Any): The updated value from the table view
      role (int): Role for which the data is set

    Returns (bool): True when data is set, otherwise false

    """
    if index.isValid() and role in (Qt.ItemDataRole.EditRole, Qt.ItemDataRole.UserRole):
      row_index = index.row()
      column = self.data_name_map.get(index.column())
      self.data_set[row_index][column] = value
      self.dataChanged.emit(index, index, role)
      return True
    return False

  def data(self,
           index: Union[QModelIndex, QPersistentModelIndex],
           role: int = Qt.ItemDataRole) -> Any:  # type: ignore[assignment]
    """
    Gets the data from the table cell represented by the index. Data is only retrieved for the following roles:
    - DisplayRole: When the table needs to be displayed
    - EditRole: When the data in the cell is edited via line edit
    - UserRole: When the cell contains radio button and is set via required_column_delegate
    Args:
      index (Union[QModelIndex, QPersistentModelIndex]): Index of the respective table cell to retrieve data
      role (int): Role for the data operation.

    Returns (Any): String data representation if available, otherwise null-string/None

    """
    if (index.isValid() and
        role in (Qt.ItemDataRole.DisplayRole, Qt.ItemDataRole.EditRole, Qt.ItemDataRole.UserRole)):
      row = index.row()
      column = index.column()
      return self.data_set[row].get(self.data_name_map.get(column))
    else:
      return None

  def flags(self,
            index: Union[QModelIndex, QPersistentModelIndex]) -> Qt.ItemFlag:
    """
    Flags required for the table cell
    Args:
      index (Union[QModelIndex, QPersistentModelIndex]): Table cell index

    Returns (Qt.ItemFlag): The combinations of flags for the cell represented by the index

    """
    if index.isValid():
      return (Qt.ItemFlag.ItemIsEditable
              | Qt.ItemFlag.ItemIsSelectable
              | Qt.ItemFlag.ItemIsEnabled)
    return Qt.ItemFlag.NoItemFlags

  @Slot(int)
  def delete_data(self, position: int) -> None:
    """
    Slot invoked to delete the data from data set at the specific position
    Args:
      position (int): Position of the data to be deleted from the data set

    Returns: None

    """
    try:
      data_deleted = self.data_set.pop(position)
    except IndexError:
      self.logger.warning("Invalid position: {%s}", position)
      return None
    self.logger.info("Deleted (row: {%s}, data: {%s})...", position, data_deleted)
    self.layoutChanged.emit()
    return None

  @Slot(int)
  def re_order_data(self,
                    position: int) -> None:
    """
    Slot invoked to re-order the data position in the data set. Data at the position: row will be moved above in the set
    Args:
      position (int): Position of the data to be shifted up.

    Returns: None

    """
    try:
      data_to_be_pushed = self.data_set.pop(position)
    except IndexError:
      self.logger.warning("Invalid position: {%s}", position)
      return None
    shift_position = position - 1
    shift_position = max(shift_position, 0)
    self.data_set.insert(shift_position, data_to_be_pushed)
    self.logger.info("Reordered the data, Actual position: {%s}, "
                     "New Position: {%s}, "
                     "data: {%s})", position,
                     shift_position,
                     data_to_be_pushed)
    self.layoutChanged.emit()
    return None

  def add_data_row(self) -> None:
    """
    Add an empty row to the table data set
    Returns: None

    """
    if self.data_set is not None:
      self.logger.info("Added new row...")
      self.data_set.insert(len(self.data_set), {})
      self.layoutChanged.emit()
    return None



""" Represents a project model object. """
#  PASTA-ELN and all its sub-parts are covered by the MIT license.
#
#  Copyright (c) 2024
#
#  Author: Jithu Murugan
#  Filename: project_model.py
#
#  You should have received a copy of the license with this file. Please refer the license file for more information.
from pasta_eln.dataverse.base_model import BaseModel
from pasta_eln.dataverse.incorrect_parameter_error import IncorrectParameterError


class ProjectModel(BaseModel):
  """
  Represents a project model object.

  This class inherits from the BaseModel class and provides additional properties and methods for managing project-related information.

  Args:
      _id (str | None): The ID of the object. Defaults to None.
      _rev (str | None): The revision of the object. Defaults to None.
      name (str | None): The name of the project. Defaults to None.
      comment (str | None): The comment associated with the project. Defaults to None.
      user (str | None): The user associated with the project. Defaults to None.
      date (str | None): The date of the project. Defaults to None.
      status (str | None): The status of the project. Defaults to None.
      objective (str | None): The objective of the project. Defaults to None.

  Raises:
      IncorrectParameterError: If any of the parameters are not of type str or None.

  Attributes:
      id (str | None): The ID of the object.
      rev (str | None): The revision of the object.
      name (str | None): The name of the project.
      comment (str | None): The comment associated with the project.
      user (str | None): The user associated with the project.
      date (str | None): The date of the project.
      status (str | None): The status of the project.
      objective (str | None): The objective of the project.

  """

  def __init__(self,
               _id: str | None = None,
               _rev: str | None = None,
               name: str | None = None,
               comment: str | None = None,
               user: str | None = None,
               date: str | None = None,
               status: str | None = None,
               objective: str | None = None) -> None:
    """
    Initializes a project model object.

    Args:
        _id (str | None): The ID of the object. Defaults to None.
        _rev (str | None): The revision of the object. Defaults to None.
        name (str | None): The name of the project. Defaults to None.
        comment (str | None): The comment associated with the project. Defaults to None.
        user (str | None): The user associated with the project. Defaults to None.
        date (str | None): The date of the project. Defaults to None.
        status (str | None): The status of the project. Defaults to None.
        objective (str | None): The objective of the project. Defaults to None.

    Raises:
        IncorrectParameterError: If any of the parameters are not of type str or None.

    """
    super().__init__(_id, _rev)
    if isinstance(name, str | None):
      self._name: str | None = name
    else:
      raise IncorrectParameterError(f"Expected string type for name but got {type(name)}")
    if isinstance(comment, str | None):
      self._comment: str | None = comment
    else:
      raise IncorrectParameterError(f"Expected string type for comment but got {type(comment)}")
    if isinstance(user, str | None):
      self._user: str | None = user
    else:
      raise IncorrectParameterError(f"Expected string type for user but got {type(user)}")
    if isinstance(date, str | None):
      self._date: str | None = date
    else:
      raise IncorrectParameterError(f"Expected string type for date but got {type(date)}")
    if isinstance(status, str | None):
      self._status: str | None = status
    else:
      raise IncorrectParameterError(f"Expected string type for status but got {type(status)}")
    if isinstance(objective, str | None):
      self._objective: str | None = objective
    else:
      raise IncorrectParameterError(f"Expected string type for objective but got {type(objective)}")

  @property
  def name(self) -> str | None:
    """
    Returns the name of the project.

    Returns:
        str | None: The name of the project.

    """
    return self._name

  @name.setter
  def name(self, value: str | None) -> None:
    """
    Sets the name of the project.

    Args:
        value (str | None): The name value to be set.

    Raises:
        IncorrectParameterError: If the value is not of type str.

    """
    if isinstance(value, str | None):
      self._name = value
    else:
      raise IncorrectParameterError(f"Expected string type for name but got {type(value)}")

  @name.deleter
  def name(self) -> None:
    """
    Deletes the name of the project.

    """
    del self._name

  @property
  def comment(self) -> str | None:
    """
    Returns the comment associated with the project.

    Returns:
        str | None: The comment associated with the project.

    """
    return self._comment

  @comment.setter
  def comment(self, value: str | None) -> None:
    """
    Sets the comment associated with the project.

    Args:
        value (str | None): The comment value to be set.

    Raises:
        IncorrectParameterError: If the value is not of type str.

    """
    if isinstance(value, str | None):
      self._comment = value
    else:
      raise IncorrectParameterError(f"Expected string type for comment but got {type(value)}")

  @comment.deleter
  def comment(self) -> None:
    """
    Deletes the comment associated with the project.

    """
    del self._comment

  @property
  def user(self) -> str | None:
    """
    Returns the user associated with the project.

    Returns:
        str | None: The user associated with the project.

    """
    return self._user

  @user.setter
  def user(self, value: str | None) -> None:
    """
    Sets the user associated with the project.

    Args:
        value (str | None): The user value to be set.

    Raises:
        IncorrectParameterError: If the value is not of type str.

    """
    if isinstance(value, str | None):
      self._user = value
    else:
      raise IncorrectParameterError(f"Expected string type for user but got {type(value)}")

  @user.deleter
  def user(self) -> None:
    """
    Deletes the user associated with the project.

    """
    del self._user

  @property
  def date(self) -> str | None:
    """
    Returns the date of the project.

    Returns:
        str | None: The date of the project.

    """
    return self._date

  @date.setter
  def date(self, value: str | None) -> None:
    """
    Sets the date of the project.

    Args:
        value (str | None): The date value to be set.

    Raises:
        IncorrectParameterError: If the value is not of type str.

    """
    if isinstance(value, str | None):
      self._date = value
    else:
      raise IncorrectParameterError(f"Expected string type for date but got {type(value)}")

  @date.deleter
  def date(self) -> None:
    """
    Deletes the date of the project.

    """
    del self._date

  @property
  def status(self) -> str | None:
    """
    Returns the status of the project.

    Returns:
        str | None: The status of the project.

    """
    return self._status

  @status.setter
  def status(self, value: str | None) -> None:
    """
    Sets the status of the project.

    Args:
        value (str | None): The status value to be set.

    Raises:
        IncorrectParameterError: If the value is not of type str.

    """
    if isinstance(value, str | None):
      self._status = value
    else:
      raise IncorrectParameterError(f"Expected string type for status but got {type(value)}")

  @status.deleter
  def status(self) -> None:
    """
    Deletes the status of the project.

    """
    del self._status

  @property
  def objective(self) -> str | None:
    """
    Returns the objective of the project.

    Returns:
        str | None: The objective of the project.

    """
    return self._objective

  @objective.setter
  def objective(self, value: str | None) -> None:
    """
    Sets the objective of the project.

    Args:
        value (str | None): The objective value to be set.

    Raises:
        IncorrectParameterError: If the value is not of type str.

    """
    if isinstance(value, str | None):
      self._objective = value
    else:
      raise IncorrectParameterError(f"Expected string type for objective but got {type(value)}")

  @objective.deleter
  def objective(self) -> None:
    """
    Deletes the objective of the project.

    """
    del self._objective



""" Communication class that sends signals between widgets, incl. backend"""
from typing import Any, Optional
from PySide6.QtCore import QObject, Signal   # pylint: disable=no-name-in-module
from PySide6.QtWidgets import QProgressBar # pylint: disable=no-name-in-module
from .backend import Backend

class Communicate(QObject):
  """ Communication class that sends signals between widgets, incl. backend"""
  def __init__(self, backend:Backend):
    super().__init__()
    self.backend = backend
    self.dbInfo:dict[str,Any]  = {}
    self.projectID = ''
    self.progressBar:Optional[QProgressBar] = None

  # Signals: specify emitter and receiver
  # BE SPECIFIC ABOUT WHAT THIS ACTION DOES
  changeSidebar = Signal(str)            # redraw sidebar after hide/show of project in table, focus on this projectID
  changeTable   = Signal(str, str)       # send doctype,projectID from sidebar to main-table
                                         #      can also be used for hiding the details on right side if nothing to show
  changeDetails = Signal(str)            # send docID from main-table to details
                                         #      docID (str): document-id; ''=draw nothing; 'redraw' implies redraw
  changeProject = Signal(str, str)       # send docID,projectID from sidebar or main-table to projects
  formDoc       = Signal(dict)           # send doc from details to new/edit dialog: dialogForm
  testExtractor = Signal()               # execute extractorTest in widgetDetails
  stopSequentialEdit = Signal()          # in sequential edit, stop if there is a cancel



""" MandatoryColumnDelegate  module used for the table views """
#  PASTA-ELN and all its sub-parts are covered by the MIT license.
#
#  Copyright (c) 2023
#
#  Author: Jithu Murugan
#  Filename: mandatory_column_delegate.py
#
#  You should have received a copy of the license with this file. Please refer the license file for more information.

import logging
from typing import Union

from PySide6.QtCore import QAbstractItemModel, QEvent, QModelIndex, QPersistentModelIndex, QRect, Qt
from PySide6.QtGui import QPainter
from PySide6.QtWidgets import QApplication, QRadioButton, QStyle, QStyleOptionButton, QStyleOptionViewItem, \
  QStyledItemDelegate, QWidget


class MandatoryColumnDelegate(QStyledItemDelegate):
  """
  Delegate for creating the radio buttons for the mandatory column in data hierarchy editor tables
  """

  def __init__(self) -> None:
    """
      Constructor
    """
    super().__init__()
    self.logger = logging.getLogger(f"{__name__}.{self.__class__.__name__}")

  def paint(self,
            painter: QPainter,
            option: QStyleOptionViewItem,
            index: Union[QModelIndex, QPersistentModelIndex]) -> None:
    """
    Draws the mandatory radio button within the cell represented by index
    Args:
      painter (QPainter): Painter instance for painting the button.
      option (QStyleOptionViewItem): Style option for the cell represented by index.
      index (Union[QModelIndex, QPersistentModelIndex]): Table cell index.

    Returns: None

    """
    widget = option.widget  # type: ignore[attr-defined]
    style = widget.style() if widget else QApplication.style()
    opt = QStyleOptionButton()
    radio_button = QRadioButton()
    option_rect = option.rect  # type: ignore[attr-defined]
    opt.rect = QRect(option_rect.left() + option_rect.width() / 2 - 10,  # type: ignore[attr-defined]
                     option_rect.top(),
                     option_rect.width(),
                     option_rect.height())
    is_mandatory = bool(index.data(Qt.ItemDataRole.UserRole))
    opt.state = QStyle.StateFlag.State_On if is_mandatory else QStyle.StateFlag.State_Off  # type: ignore[attr-defined]
    style.drawControl(QStyle.ControlElement.CE_RadioButton, opt, painter, radio_button)

  def editorEvent(self,
                  event: QEvent,
                  model: QAbstractItemModel,
                  option: QStyleOptionViewItem,
                  index: Union[QModelIndex, QPersistentModelIndex]) -> bool:
    """
    In case of mouse click event, the model data is toggled for the respective table cell index
    Args:
      event (QEvent): The editor event information.
      model (QAbstractItemModel): Model data representing the table view.
      option (QStyleOptionViewItem): QStyleOption for the table cell.
      index (Union[QModelIndex, QPersistentModelIndex]): Table cell index.

    Returns (bool): True/False

    """
    if event.type() == QEvent.Type.MouseButtonRelease:
      model.setData(index, not bool(index.data(Qt.ItemDataRole.UserRole)), Qt.ItemDataRole.UserRole)
    return super().editorEvent(event, model, option, index)

  def createEditor(self,
                   parent: QWidget,
                   option: QStyleOptionViewItem,
                   index: Union[QModelIndex, QPersistentModelIndex]) -> QWidget:
    """
    Disable the editor for the whole mandatory column by simply returning None
    Args:
      parent (QWidget): Parent table view.
      option (QStyleOptionViewItem): Style option for the cell represented by index.
      index (Union[QModelIndex, QPersistentModelIndex]): Cell index.

    Returns: None

    """
    return None  # type: ignore[return-value]



""" Table view model used for the metadata table in the data hierarchy editor """
#  PASTA-ELN and all its sub-parts are covered by the MIT license.
#
#  Copyright (c) 2023
#
#  Author: Jithu Murugan
#  Filename: metadata_tableview_data_model.py
#
#  You should have received a copy of the license with this file. Please refer the license file for more information.

import logging
from typing import Any, Union

from PySide6.QtCore import QModelIndex, QPersistentModelIndex, Qt
from PySide6.QtWidgets import QWidget

from .constants import METADATA_TABLE_LIST_COLUMN_INDEX
from .tableview_data_model import TableViewModel


class MetadataTableViewModel(TableViewModel):
  """
  Data-model for the data hierarchy metadata table view
  """

  def __init__(self,
               parent: Union[QWidget | None] = None):
    """
    Initialize the data model representing the metadata of a type in the data hierarchy document
    Args:
      parent (QWidget): Parent view or the widget
    """
    super().__init__(parent)
    self.logger = logging.getLogger(f"{__name__}.{self.__class__.__name__}")
    self.data_set = []
    self.data_name_map = {
      0: "name",
      1: "query",
      2: "list",
      3: "unit",
      4: "IRI",
      5: "mandatory",
      6: "delete",
      7: "re-order"
    }
    self.header_values: list[str] = list(self.data_name_map.values())
    self.column_widths: dict[int, int] = {
      0: 100,  # Name column width
      1: 300,  # Query column width
      2: 150,  # list column width
      3: 100,  # unit column width
      4: 100,  # IRI column width
      5: 130,  # mandatory column width
      6: 120,  # delete column width
      7: 120  # re-order column width
    }
    self.columns_count: int = len(self.header_values)

  def setData(self,
              index: Union[QModelIndex, QPersistentModelIndex],
              value: Any, role: int = Qt.ItemDataRole) -> bool:  # type: ignore[assignment]
    """
    Overridden method for setting the data, only handling the METADATA_TABLE_LIST_COLUMN_INDEX, the rest is handled by base class
    Args:
      index (Union[QModelIndex, QPersistentModelIndex]): Table cell index
      value (Any): New value
      role (int): Role for the data

    Returns: Nothing
    """
    if (index.isValid() and
        index.column() == METADATA_TABLE_LIST_COLUMN_INDEX):
      value = [i.strip() for i in value.split(',')] \
        if value and ',' in value \
        else value or []
    return super().setData(index, value, role)

  def data(self,
           index: Union[QModelIndex, QPersistentModelIndex],
           role: int = Qt.ItemDataRole) -> Any:  # type: ignore[assignment]
    """
    Overridden method for getting the data, only handles the METADATA_TABLE_LIST_COLUMN_INDEX case
    Args:
      index (Union[QModelIndex, QPersistentModelIndex]): Table cell index
      role (int): Role for the data

    Returns: Data retrieved for the respective index

    """
    if (index.isValid()
        and index.column() == METADATA_TABLE_LIST_COLUMN_INDEX):
      data = super().data(index, role)
      return ",".join(data) \
        if isinstance(data, list) \
        else data
    return super().data(index, role)



""" Represents the completed uploads UI. """
#  PASTA-ELN and all its sub-parts are covered by the MIT license.
#
#  Copyright (c) 2024
#
#  Author: Jithu Murugan
#  Filename: completed_uploads.py
#
#  You should have received a copy of the license with this file. Please refer the license file for more information.
import logging
import textwrap
from typing import Any

from PySide6 import QtCore
from PySide6.QtWidgets import QDialog, QFrame

from pasta_eln.GUI.dataverse.completed_upload_task import Ui_CompletedUploadTaskFrame
from pasta_eln.GUI.dataverse.completed_uploads_base import Ui_CompletedUploadsForm
from pasta_eln.dataverse.database_api import DatabaseAPI
from pasta_eln.dataverse.upload_model import UploadModel
from pasta_eln.dataverse.upload_status_values import UploadStatusValues
from pasta_eln.dataverse.utils import get_formatted_dataverse_url


class CompletedUploads(Ui_CompletedUploadsForm):
  """
  Represents the completed uploads UI.

  Explanation:
      This class handles the completed uploads UI, including loading and clearing the UI,
      and creating the completed upload task widget.
  """

  def __new__(cls, *_: Any, **__: Any) -> Any:
    """
    Creates a new instance of the CompletedUploads class.

    Explanation:
        This method creates a new instance of the CompletedUploads class.

    Args:
        *_: Variable length argument list.
        **__: Arbitrary keyword arguments.

    Returns:
        Any: The new instance of the CompletedUploads class.
    """
    return super(CompletedUploads, cls).__new__(cls)

  def __init__(self) -> None:
    """
    Initializes a new instance of the CompletedUploads class.

    Explanation:
        This method initializes a new instance of the CompletedUploads class.
        It sets up the logger, creates a QDialog instance, and sets the window modality.
    """
    self.load_complete: bool = False
    self.next_bookmark: str | None = None
    self.logger = logging.getLogger(f"{__name__}.{self.__class__.__name__}")
    self.instance = QDialog()
    super().setupUi(self.instance)
    self.db_api = DatabaseAPI()
    self.instance.setWindowModality(QtCore.Qt.WindowModality.ApplicationModal)
    self.completedUploadsScrollArea.verticalScrollBar().valueChanged.connect(self.scrolled)
    self.filterTermLineEdit.textChanged.connect(self.load_ui)

  def load_ui(self) -> None:
    """
    Loads the UI for completed uploads.

    Explanation:
        This method loads the UI for completed uploads by clearing the existing UI and adding widgets for each upload.
        It queries the completed uploads from the database and creates the corresponding widgets.

    Args:
        self: Represents the instance of the class.
    """
    self.logger.info("Loading completed uploads..")
    self.clear_ui()
    result = self.db_api.get_paginated_models(UploadModel, filter_term=self.filterTermLineEdit.text())
    self.next_bookmark = result["bookmark"] if isinstance(result["bookmark"], str) else None
    if self.next_bookmark is None:
      self.logger.error("Unable to retrieve models, invalid bookmark returned!")
      return
    for upload in result["models"]:
      if isinstance(upload, UploadModel):
        widget = self.get_completed_upload_task_widget(upload)
        self.completedUploadsVerticalLayout.addWidget(widget)
      else:
        self.logger.error("Incorrect type in queried models!")

  def clear_ui(self) -> None:
    """
    Clears the UI for completed uploads.

    Explanation:
        This method clears the UI for completed uploads by removing all the widgets from the layout.
    """
    for widget_pos in reversed(range(self.completedUploadsVerticalLayout.count())):
      self.completedUploadsVerticalLayout.itemAt(widget_pos).widget().setParent(None)

  def get_completed_upload_task_widget(self, upload: UploadModel) -> QFrame:
    """
    Retrieves the completed upload task widget.

    Explanation:
        This function retrieves the completed upload task widget for the given upload.
        It creates a QFrame instance and sets up the UI for the completed upload task.

    Args:
        upload (UploadModel): The upload model representing the completed upload.

    Returns:
        QFrame: The completed upload task widget.
    """
    completed_task_frame = QFrame()
    completed_task_ui = Ui_CompletedUploadTaskFrame()
    completed_task_ui.setupUi(completed_task_frame)
    completed_task_ui.projectNameLabel.setText(textwrap.fill(upload.project_name or "", 45, max_lines=1))
    completed_task_ui.projectNameLabel.setToolTip(
      f"{completed_task_ui.projectNameLabel.toolTip()}\n{upload.project_name}")
    completed_task_ui.statusLabel.setText(upload.status)

    match upload.status:
      case UploadStatusValues.Queued.name | UploadStatusValues.Uploading.name:
        self.set_completed_task_properties(
          completed_task_ui,
          "Waiting..",
          "",
          "Waiting..",
          "Waiting..")
      case UploadStatusValues.Finished.name:
        url_tooltip = f"{completed_task_ui.dataverseUrlLabel.toolTip()}\n{upload.dataverse_url}"

        self.set_completed_task_properties(
          completed_task_ui,
          get_formatted_dataverse_url(upload.dataverse_url or ""),
          url_tooltip,
          upload.created_date_time or "",
          upload.finished_date_time or "")
      case UploadStatusValues.Cancelled.name:
        self.set_completed_task_properties(
          completed_task_ui,
          "NA",
          "",
          "NA",
          "NA")
      case _:  # Error or Warning
        self.set_completed_task_properties(
          completed_task_ui,
          "Error state..",
          "",
          "Error state..",
          "Error state..")
    return completed_task_frame

  def set_completed_task_properties(self,
                                    completed_task_ui: Ui_CompletedUploadTaskFrame,
                                    dataverse_url: str = "",
                                    dataverse_url_tooltip: str = "",
                                    started_date_time: str = "",
                                    finished_date_time: str = "") -> None:
    """
    Sets the properties of the completed task UI.

    Explanation:
        This function updates the properties of the completed task UI based on the provided values.
        It sets the dataverse URL, tooltip, start date time, and finish date time for the task.

    Args:
        completed_task_ui (Ui_CompletedUploadTaskFrame): The UI frame for the completed task.
        dataverse_url (str): The dataverse URL for the task.
        dataverse_url_tooltip (str): The tooltip for the dataverse URL.
        started_date_time (str): The start date and time of the task.
        finished_date_time (str): The finish date and time of the task.
    """

    (completed_task_ui.dataverseUrlLabel
     .setToolTip(dataverse_url_tooltip or completed_task_ui.dataverseUrlLabel.toolTip()))
    (completed_task_ui.dataverseUrlLabel
     .setText(dataverse_url or completed_task_ui.dataverseUrlLabel.text()))
    (completed_task_ui.startedDateTimeLabel
     .setText(started_date_time or completed_task_ui.startedDateTimeLabel.text()))
    (completed_task_ui.finishedDateTimeLabel
     .setText(finished_date_time or completed_task_ui.finishedDateTimeLabel.text()))

  def show(self) -> None:
    """
    Shows the completed uploads dialog.

    Explanation:
        This method displays the completed uploads dialog by loading the UI and showing the dialog instance.

    Args:
        self: Represents the instance of the class.
    """

    self.logger.info("Showing completed uploads..")
    self.load_complete = False
    self.load_ui()
    self.instance.show()

  def scrolled(self, scroll_value: int) -> None:
    """
    Scrolled event handler for the completed uploads verticalScrollBar.

    Explanation:
        This function handles scrolling through the completed uploads based on the scroll value provided.
        It loads more data if the scroll reaches the maximum and there is a next bookmark available.

    Args:
        scroll_value (int): The value of the scroll position.
    """
    if self.load_complete:
      self.logger.info("Data load completed, hence skipping..")
      return
    vertical_scroll_bar = self.completedUploadsScrollArea.verticalScrollBar()
    if scroll_value == vertical_scroll_bar.maximum() and self.next_bookmark:
      result = self.db_api.get_paginated_models(UploadModel,
                                                filter_term=self.filterTermLineEdit.text(),
                                                bookmark=self.next_bookmark)
      bookmark = result["bookmark"] if isinstance(result["bookmark"], str) else None
      if self.next_bookmark != bookmark:
        self.next_bookmark = bookmark
        for upload in result["models"]:
          if isinstance(upload, UploadModel):
            widget = self.get_completed_upload_task_widget(upload)
            self.completedUploadsVerticalLayout.addWidget(widget)
      else:
        self.load_complete = True



""" renders each leaf of project tree using QPaint """
import base64, logging, re
from typing import Optional, Any
from PySide6.QtCore import Qt, QSize, QPoint, QMargins, QRectF, QModelIndex# pylint: disable=no-name-in-module
from PySide6.QtGui import QStaticText, QPixmap, QTextDocument, QPainter, QColor, QPen # pylint: disable=no-name-in-module
from PySide6.QtWidgets import QStyledItemDelegate, QStyleOptionViewItem # pylint: disable=no-name-in-module
from PySide6.QtSvg import QSvgRenderer                        # pylint: disable=no-name-in-module
from ..guiCommunicate import Communicate
from ..guiStyle import getColor
from ..miscTools import markdownStyler

_DO_NOT_RENDER_ = ['image','content','metaVendor','metaUser','shasum','comment']

class ProjectLeafRenderer(QStyledItemDelegate):
  """ renders each leaf of project tree using QPaint """
  def __init__(self, comm:Communicate) -> None:
    super().__init__()
    self.comm = comm
    self.debugMode = logging.root.level<logging.INFO
    self.widthImage = self.comm.backend.configuration['GUI']['imageWidthProject']
    self.widthContent = self.comm.backend.configuration['GUI']['widthContent']
    self.docTypeOffset = self.comm.backend.configuration['GUI']['docTypeOffset']
    self.frameSize = self.comm.backend.configuration['GUI']['frameSize']
    self.maxHeight = self.comm.backend.configuration['GUI']['maxProjectLeafHeight']
    self.lineSep = 20
    self.penDefault:Optional[QPen] = None
    self.penHighlight              = QPen(QColor(getColor(self.comm.backend, 'primary')))
    self.penHighlight.setWidth(2)
    self.colorMargin1 = QColor(getColor(self.comm.backend, 'secondary')).darker(110)
    self.colorMargin2 = QColor(getColor(self.comm.backend, 'secondaryLight'))


  def paint(self, painter:QPainter, option:QStyleOptionViewItem, index:QModelIndex) -> None:                 # type: ignore
    """
    Paint this item
    - coordinates: left, top
    - COS top left

    Args:
      painter (QPainter): painter
      option (QStyleOptionViewItem): option incl. current coordinates
      index (QModelIndex): index
    """
    name = index.data(Qt.ItemDataRole.DisplayRole)
    data = index.data(Qt.ItemDataRole.UserRole+1)
    if not data or data['hierStack'] is None or self.comm is None:
      return
    docID   = data['hierStack'].split('/')[-1]
    # GUI
    if self.penDefault is None:
      self.penDefault = QPen(painter.pen())
    x0, y0 = option.rect.topLeft().toTuple()                                                                  # type: ignore[attr-defined]
    widthContent = min(self.widthContent,  \
                       int((option.rect.bottomRight()-option.rect.topLeft()).toTuple()[0]/2) )                # type: ignore[attr-defined]
    docTypeOffset = min(self.docTypeOffset, \
                        int((option.rect.bottomRight()-option.rect.topLeft()).toTuple()[0]/3.5) )             # type: ignore[attr-defined]
    bottomRight2nd = option.rect.bottomRight()- QPoint(self.frameSize+1,self.frameSize)                       # type: ignore[attr-defined]
    painter.fillRect(option.rect.marginsRemoved(QMargins(2,6,4,0)),  self.colorMargin1)                       # type: ignore[attr-defined]
    if data['docType'][0][0]=='x':
      painter.fillRect(option.rect.marginsRemoved(QMargins(-2,3,8,5)), self.colorMargin2.darker(102))         # type: ignore[attr-defined]
    else:
      painter.fillRect(option.rect.marginsRemoved(QMargins(-2,3,8,5)), self.colorMargin2.lighter(210))        # type: ignore[attr-defined]
    # header
    y = self.lineSep/2
    docTypeText= '/'.join(data['docType'])
    if data['docType'][0][0]=='x':
      docTypeText = self.comm.backend.db.dataHierarchy['x1']['title'].lower()[:-1]
    nameText = name if len(name)<55 else '...'+name[-50:]
    if not data['gui'][0]:  #Only draw first line
      staticText = QStaticText(f'<strong>{nameText}</strong>')
      staticText.setTextWidth(docTypeOffset)
      painter.drawStaticText(x0, y0+y, staticText)
      painter.drawStaticText(x0+docTypeOffset, y0+y, QStaticText(docTypeText))
      return
    # details = body
    doc     = self.comm.backend.db.getDoc(docID)
    if len(doc)<2:
      print(f'**ERROR cannot read docID: {docID}')
      logging.error('LeafRenderer: Cannot read docID %s',docID)
      return
    hiddenText = ('     \U0001F441' if [b for b in doc['-branch'] if False in b['show']] else '')
    staticText = QStaticText(f'<strong>{nameText} {hiddenText}</strong>')
    staticText.setTextWidth(docTypeOffset)
    painter.drawStaticText(x0, y0+y, staticText)
    painter.drawStaticText(x0+docTypeOffset, y0+y, QStaticText(docTypeText))
    if self.debugMode:
      painter.drawStaticText(x0+700, y0+y, QStaticText(data['hierStack']))
    width, height = -1, -1
    if doc['-tags']:
      y += self.lineSep
      tags = ['_curated_' if i=='_curated' else f'#{i}' for i in doc['-tags']]
      tags = ['\u2605'*int(i[2]) if i[:2]=='#_' else i for i in tags]
      painter.drawStaticText(x0, y0+y, QStaticText(f'Tags: {" ".join(tags)}'))
    for key in doc:
      if key in _DO_NOT_RENDER_ or key[0] in ['-','_']:
        continue
      y += self.lineSep
      if isinstance(doc[key], str):
        value = ''
        if re.match(r'^[a-z\-]-[a-z0-9]{32}$',doc[key]) is None:  #normal text
          value = doc[key]
        elif self.comm is not None:                     #link
          table = self.comm.backend.db.getView(f'viewDocType/{key}All')
          choices= [i for i in table if i['id']==doc[key]]
          if len(choices)==1:
            value = '\u260D '+choices[0]['value'][0]
          else:
            value = 'ERROR WITH LINK'
        painter.drawStaticText(x0, y0+y, QStaticText(f'{key}: {value}'))
      elif isinstance(doc[key], list):                     #list of qrCodes
        painter.drawStaticText(x0, y0+y, QStaticText(f'{key}: ' + ', '.join([str(i) for i in doc[key]])))
    for textType in ['comment', 'content']:
      if textType in doc and (textType != 'content' or 'image' not in doc or doc['image'] == ''):
        textDoc = QTextDocument()
        textDoc.setMarkdown(markdownStyler(doc[textType]))
        if textType == 'comment':
          textDoc.setTextWidth(bottomRight2nd.toTuple()[0]-x0-widthContent-2*self.frameSize)
          width, height = textDoc.size().toTuple() # type: ignore
          painter.translate(QPoint(x0-3, y0+y+15))
          yMax = int(self.maxHeight-2*self.frameSize-y-15)
        else:
          textDoc.setTextWidth(widthContent)
          width, height = textDoc.size().toTuple() # type: ignore
          topLeftContent = option.rect.topRight() - QPoint(width+self.frameSize-2,-self.frameSize)           # type: ignore[attr-defined]
          painter.translate(topLeftContent)
          yMax = int(self.maxHeight-3*self.frameSize)
          y = 0
        textDoc.drawContents(painter, QRectF(0, 0, width, yMax))
        if y+height > self.maxHeight-2*self.frameSize:
          painter.setPen(self.penHighlight)
          painter.drawLine(self.frameSize, yMax+self.frameSize, width-self.frameSize, yMax+self.frameSize)
          painter.setPen(self.penDefault)
        if textType == 'comment':
          painter.translate(-QPoint(x0-3, y0+y+15))
        else:
          topLeftContent = option.rect.topRight() - QPoint(width+self.frameSize-2,-self.frameSize)           # type: ignore[attr-defined]
          painter.translate(-topLeftContent)
    if 'image' in doc and doc['image']!='':
      if doc['image'].startswith('data:image/'):
        pixmap = self.imageFromDoc(doc)
        width2nd = min(self.widthImage, pixmap.width()+self.frameSize)
        topLeft2nd     = option.rect.topRight()   - QPoint(width2nd+self.frameSize+1,-self.frameSize)        # type: ignore[attr-defined]
        painter.drawPixmap(topLeft2nd, pixmap)
      elif doc['image'].startswith('<?xml'):
        topLeft2nd     = option.rect.topRight()   - QPoint(self.widthImage+self.frameSize+1,-self.frameSize) # type: ignore[attr-defined]
        image = QSvgRenderer(bytearray(doc['image'], encoding='utf-8'))
        image.render(painter,    QRectF(topLeft2nd, bottomRight2nd))
    return


  def sizeHint(self, option:QStyleOptionViewItem, index:QModelIndex) -> QSize:                               # type: ignore
    """
    determine size of this leaf
    """
    if not index or not index.data(Qt.ItemDataRole.UserRole+1):
      return QSize()
    hierStack = index.data(Qt.ItemDataRole.UserRole+1)['hierStack']
    if hierStack is None or self.comm is None:
      return QSize()
    if not index.data(Qt.ItemDataRole.UserRole+1)['gui'][0]:
      return QSize(400, self.lineSep*2)
    docID   = hierStack.split('/')[-1]
    doc = self.comm.backend.db.getDoc(docID)
    if len(doc)<2:
      if len(self.comm.backend.db.getDoc(hierStack.split('/')[0]))>2: #only refresh when project still exists
        self.comm.changeProject.emit('','')
      return QSize()
    docKeys = doc.keys()
    widthContent = min(self.widthContent,  \
                       int((option.rect.bottomRight()-option.rect.topLeft()).toTuple()[0]/2) )               # type: ignore[attr-defined]
    height  = len([i for i in docKeys if not i in _DO_NOT_RENDER_ and i[0] not in ['-','_'] ])  #height in text lines
    height += 1 if '-tags' in docKeys and len(doc['-tags']) > 0 else 0
    height  = (height+3) * self.lineSep
    if 'content' in docKeys:
      text = QTextDocument()
      text.setMarkdown(doc['content'])
      text.setTextWidth(widthContent)
      height = max(height, text.size().toTuple()[1]) +2*self.frameSize # type: ignore
    elif 'image' in docKeys:
      if doc['image'].startswith('data:image/'):
        pixmap = self.imageFromDoc(doc)
        height = max(height, pixmap.height())+2*self.frameSize
      else:
        height = max(height, int(self.widthImage*3/4))+2*self.frameSize
    elif 'comment' in doc.keys() and len(doc['comment'])>0:
      text = QTextDocument()
      comment = doc['comment']
      text.setMarkdown(comment.strip())
      text.setTextWidth(widthContent)
      height += text.size().toTuple()[1] # type: ignore
      height -= 25
    else:
      height -= 25
    return QSize(400, min(height, self.maxHeight))


  def imageFromDoc(self, doc:dict[str,Any]) -> QPixmap:
    """ Create image from image in doc

    Args:
      doc (dict): document

    Returns:
      QPixmap: image
    """
    result = QPixmap()
    result.loadFromData(base64.b64decode(doc['image'][22:]))
    result = result.scaledToWidth(self.widthImage)
    if result.height()>self.maxHeight:
      result = result.scaledToHeight(self.maxHeight-self.frameSize*2)
    return result



"""extract data from vendor
- javascript object notion
"""
import json
import os

def use(filePath, recipe='', saveFileName=None):
  """
  Args:
    filePath (Path): full path file name
    recipe (string): supplied to guide recipes
                     recipe is / separated hierarchical elements parent->child
    saveFileName (string): if given, save the image to this file-name

  Returns:
    dict: containing image, metaVendor, metaUser, recipe
  """
  # Extractor for fancy instrument
  content = ''
  fileSize = os.stat(filePath).st_size / (1024*1024)  #in MB
  if fileSize<0.3:
    with open(filePath,'r', encoding='utf-8') as jsonFile:
      jsonContent = jsonFile.read()
      content = json.loads(jsonContent)
      if not isinstance(content, dict):
        content= {'content': json.loads(jsonContent)}
    content= f'```json\n{json.dumps(content, indent=2)}\n```'
  else:
    content= 'Too large json file'
  recipe = 'procedure/json'

  # return everything
  return {'content':content, 'recipe':recipe, 'metaVendor':{}, 'metaUser':{}}

  #other datatypes follow here
  #...
  #final return if nothing successful
  #return {'recipe': '-', 'metaVendor':{}, 'metaUser':{}}



""" CreateTypeDialog used for the create type dialog """
#  PASTA-ELN and all its sub-parts are covered by the MIT license.
#
#  Copyright (c) 2023
#
#  Author: Jithu Murugan
#  Filename: create_type_dialog.py
#
#  You should have received a copy of the license with this file. Please refer the license file for more information.

import logging
from collections.abc import Callable
from typing import Any

from PySide6 import QtCore
from PySide6.QtCore import QRegularExpression
from PySide6.QtGui import QRegularExpressionValidator
from PySide6.QtWidgets import QDialog

from pasta_eln.GUI.data_hierarchy.create_type_dialog_base import Ui_CreateTypeDialogBase


class CreateTypeDialog(Ui_CreateTypeDialogBase):
  """
  Abstracted dialog for the create type
  """

  def __new__(cls, *_: Any, **__: Any) -> Any:
    """
    Instantiates the create type dialog
    """
    return super(CreateTypeDialog, cls).__new__(cls)

  def __init__(self,
               accepted_callback: Callable[[], None],
               rejected_callback: Callable[[], None]) -> None:
    """
    Initializes the create type dialog
    Args:
      accepted_callback (Callable): Accepted button parent callback.
      rejected_callback (Callable): Rejected button parent callback.
    """
    self.logger = logging.getLogger(f"{__name__}.{self.__class__.__name__}")
    self.next_struct_level: str | None = ""
    self.instance = QDialog()
    super().setupUi(self.instance)
    # Restricts the title input to allow anything except x or space
    # as the first character which is reserved for structural level
    self.titleLineEdit.setValidator(QRegularExpressionValidator(QRegularExpression("^[^ Ax].*")))
    self.setup_slots(accepted_callback, rejected_callback)

  def setup_slots(self,
                  accepted_callback: Callable[[], None],
                  rejected_callback: Callable[[], None]) -> None:
    """
    Sets up the slots for the dialog
    Args:
      accepted_callback (Callable): Accepted button parent callback.
      rejected_callback (Callable): Rejected button parent callback.

    Returns: None

    """
    self.buttonBox.accepted.connect(accepted_callback)
    self.buttonBox.rejected.connect(rejected_callback)
    self.structuralLevelCheckBox.stateChanged.connect(self.structural_level_checkbox_callback)

  def structural_level_checkbox_callback(self) -> None:
    """
    Callback invoked when the state changes for structuralLevelCheckBox

    Returns: Nothing
    """
    if self.structuralLevelCheckBox.isChecked():
      self.titleLineEdit.setText(self.next_struct_level.replace("x", "Structure level ")
                                 if self.next_struct_level else "")
      self.titleLineEdit.setDisabled(True)
    else:
      self.titleLineEdit.clear()
      self.titleLineEdit.setDisabled(False)

  def show(self) -> None:
    """
    Displays the dialog

    Returns: None

    """
    self.instance.setWindowModality(QtCore.Qt.WindowModality.ApplicationModal)
    self.instance.show()

  def clear_ui(self) -> None:
    """
    Clear the Dialog UI

    Returns: Nothing

    """
    self.displayedTitleLineEdit.clear()
    self.titleLineEdit.clear()
    self.structuralLevelCheckBox.setChecked(False)

  def set_structural_level_title(self,
                                 structural_level: str | None) -> None:
    """
    Set the next possible structural type level title

    Args:
      structural_level (str): Passed in structural level of the format (x0, x1, x2 ...)

    Returns: Nothing

    """
    self.logger.info("Next structural level set: {%s}...", structural_level)
    self.next_struct_level = structural_level



# Form implementation generated from reading ui file 'completed_upload_task.ui'
#
# Created by: PyQt6 UI code generator 6.4.2
#
# WARNING: Any manual changes made to this file will be lost when pyuic6 is
# run again.  Do not edit this file unless you know what you are doing.


from PySide6 import QtCore, QtGui, QtWidgets


class Ui_CompletedUploadTaskFrame(object):
  def setupUi(self, CompletedUploadTaskFrame):
    CompletedUploadTaskFrame.setObjectName("CompletedUploadTaskFrame")
    CompletedUploadTaskFrame.resize(1288, 64)
    CompletedUploadTaskFrame.setFrameShape(QtWidgets.QFrame.Shape.StyledPanel)
    CompletedUploadTaskFrame.setFrameShadow(QtWidgets.QFrame.Shadow.Raised)
    self.horizontalLayout_2 = QtWidgets.QHBoxLayout(CompletedUploadTaskFrame)
    self.horizontalLayout_2.setObjectName("horizontalLayout_2")
    self.horizontalLayout = QtWidgets.QHBoxLayout()
    self.horizontalLayout.setObjectName("horizontalLayout")
    self.projectNameLabel = QtWidgets.QLabel(parent=CompletedUploadTaskFrame)
    self.projectNameLabel.setObjectName("projectNameLabel")
    self.horizontalLayout.addWidget(self.projectNameLabel)
    self.dataverseUrlLabel = QtWidgets.QLabel(parent=CompletedUploadTaskFrame)
    self.dataverseUrlLabel.setOpenExternalLinks(True)
    self.dataverseUrlLabel.setObjectName("dataverseUrlLabel")
    self.horizontalLayout.addWidget(self.dataverseUrlLabel)
    self.startedDateTimeLabel = QtWidgets.QLabel(parent=CompletedUploadTaskFrame)
    self.startedDateTimeLabel.setObjectName("startedDateTimeLabel")
    self.horizontalLayout.addWidget(self.startedDateTimeLabel)
    self.finishedDateTimeLabel = QtWidgets.QLabel(parent=CompletedUploadTaskFrame)
    self.finishedDateTimeLabel.setObjectName("finishedDateTimeLabel")
    self.horizontalLayout.addWidget(self.finishedDateTimeLabel)
    self.statusLabel = QtWidgets.QLabel(parent=CompletedUploadTaskFrame)
    sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Fixed, QtWidgets.QSizePolicy.Policy.Preferred)
    sizePolicy.setHorizontalStretch(0)
    sizePolicy.setVerticalStretch(0)
    sizePolicy.setHeightForWidth(self.statusLabel.sizePolicy().hasHeightForWidth())
    self.statusLabel.setSizePolicy(sizePolicy)
    self.statusLabel.setMinimumSize(QtCore.QSize(100, 0))
    self.statusLabel.setObjectName("statusLabel")
    self.horizontalLayout.addWidget(self.statusLabel)
    self.horizontalLayout_2.addLayout(self.horizontalLayout)

    self.retranslateUi(CompletedUploadTaskFrame)
    QtCore.QMetaObject.connectSlotsByName(CompletedUploadTaskFrame)

  def retranslateUi(self, CompletedUploadTaskFrame):
    _translate = QtCore.QCoreApplication.translate
    CompletedUploadTaskFrame.setWindowTitle(_translate("CompletedUploadTaskFrame", "Frame"))
    self.projectNameLabel.setToolTip(_translate("CompletedUploadTaskFrame", "PASTA project name which was uploaded to dataverse."))
    self.projectNameLabel.setText(_translate("CompletedUploadTaskFrame", "Example Project 1"))
    self.dataverseUrlLabel.setToolTip(_translate("CompletedUploadTaskFrame", "The dataverse URL where the PASTA project was uploaded."))
    self.dataverseUrlLabel.setText(_translate("CompletedUploadTaskFrame", "<html><head/><body><p>Dataverse URL: <a href=\"https://data-beta.fz-juelich.de/dataset.xhtml?persistentId=doi:10.0346/JUELICH-DATA-BETA/BORORQ\"><span style=\" font-style:italic; text-decoration: underline; color:#77767b;\">Test Data Set 12</span></a></p></body></html>"))
    self.startedDateTimeLabel.setToolTip(_translate("CompletedUploadTaskFrame", "Dataverse upload start time."))
    self.startedDateTimeLabel.setText(_translate("CompletedUploadTaskFrame", "01-16-2024 10:52:15"))
    self.finishedDateTimeLabel.setToolTip(_translate("CompletedUploadTaskFrame", "Dataverse upload finish time."))
    self.finishedDateTimeLabel.setText(_translate("CompletedUploadTaskFrame", "01-16-2024 10:52:15"))
    self.statusLabel.setToolTip(_translate("CompletedUploadTaskFrame", "Displays the status of the upload."))
    self.statusLabel.setText(_translate("CompletedUploadTaskFrame", "Queued"))


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    CompletedUploadTaskFrame = QtWidgets.QFrame()
    ui = Ui_CompletedUploadTaskFrame()
    ui.setupUi(CompletedUploadTaskFrame)
    CompletedUploadTaskFrame.show()
    sys.exit(app.exec())



#!/usr/bin/python3
"""TEST using the FULL set of python-requirements: create 3 projects; simplified form of testTutorialComplex """
import os, shutil, logging, json
import warnings
import unittest
from typing import Any
from pathlib import Path
from zipfile import ZipFile, ZIP_DEFLATED
import pytest
from pasta_eln.backend import Backend
from pasta_eln.inputOutput import exportELN, importELN
from pasta_eln.miscTools import outputString


class TestStringMethods(unittest.TestCase):
  """
  derived class for this test
  """
  def __init__(self, *args, **kwargs):
    super().__init__(*args, **kwargs)
    self.be = None
    self.dirName = ''

  @pytest.mark.skip(
    reason="Disabled for github since cannot create couchdb instance during actions")
  def test_main(self):
    """
    main function
    """
    ###### BEGIN testELNFile: copy into all export tests in this folder   ######
    def testELNFile(fileName:str, verbose:bool) -> None:
      """
      an test that is similar to the test of TheELNConsortium
      - copied here for portability

      Args:
        fileName (str): file name of .eln to test
      """
      # global variables worth discussion
      ROCRATE_NOTE_MANDATORY = ['version','sdPublisher']
      DATASET_MANDATORY = ['name']
      DATASET_SUGGESTED = ['author','mentions',  'dateCreated', 'dateModified', 'identifier', 'text', 'keywords']
      FILE_MANDATORY = ['name']
      FILE_SUGGESTED = ['sha256', 'encodingFormat', 'contentSize', 'description']

      # runtime global variables
      METADATA_FILE = 'ro-crate-metadata.json'
      OUTPUT_INFO = False
      OUTPUT_COUNTS = True
      KNOWN_KEYS = DATASET_MANDATORY+DATASET_SUGGESTED+FILE_MANDATORY+FILE_SUGGESTED+['@id', '@type']

      logJson = {}
      def processNode(graph: list[dict[str,Any]], nodeID:str) -> bool:
        """
        recursive function call to process each node

        Args:
        graph: full graph
        nodeID: id of node in graph
        """
        globalSuccess = True
        nodes = [i for i in graph if '@id' in i and i['@id'] == nodeID]
        if len(nodes)!=1:
          print('**ERROR: all entries must only occur once in crate. check:', nodeID)
          return False
        node = nodes[0]
        # CHECK IF MANDATORY AND SUGGESTED KEYWORDS ARE PRESENT
        if '@type' not in node:
          print('**ERROR: all nodes must have @type. check:', nodeID)
        if node['@type'] == 'Dataset':
          for key in DATASET_MANDATORY:
            if not key in node:
              print(f'**ERROR in dataset: "{key}" not in @id={node["@id"]}')
              globalSuccess = False
          for key in DATASET_SUGGESTED:
            if not key in node and OUTPUT_INFO and verbose:
              print(f'**INFO for dataset: "{key}" not in @id={node["@id"]}')
        elif node['@type'] == 'File':
          for key in FILE_MANDATORY:
            if not key in node:
              print(f'**ERROR in file: "{key}" not in @id={node["@id"]}')
              globalSuccess = False
          for key in FILE_SUGGESTED:
            if not key in node and OUTPUT_INFO and verbose:
              print(f'**INFO for file: "{key}" not in @id={node["@id"]}')
        # CHECK PROPERTIES FOR ALL KEYS
        if any(str(i).strip()=='' for i in node.values()):
          print(f'**WARNING: {nodeID} contains empty values in the key-value pairs', node)
        # SPECIFIC CHECKS ON CERTAIN KEYS
        if isinstance(node.get('keywords', ''), list):
          print(f'**ERROR: {nodeID} contains an array of keywords. Use comma or space separated string')
          globalSuccess = False
        # recurse to children
        children = node.pop('hasPart') if 'hasPart' in node else []
        for child in children:
          globalSuccess = processNode(graph, child['@id']) and globalSuccess
        return globalSuccess

      if verbose:
        print(f'\n\nParse: {fileName}')
      with ZipFile(fileName, 'r', compression=ZIP_DEFLATED) as elnFile:
        success = True
        metadataJsonFile = [i for i in elnFile.namelist() if i.endswith(METADATA_FILE)][0]
        with elnFile.open(metadataJsonFile) as roCrateFile:
          metadataContent = json.loads(roCrateFile.read())
        graph = metadataContent["@graph"]
        # find information from master node
        ro_crate_nodes = [i for i in graph if i["@id"] == METADATA_FILE]
        if len(ro_crate_nodes) == 1:
          for key in ROCRATE_NOTE_MANDATORY:
            if not key in ro_crate_nodes[0]:
              print(f'**ERROR: "{key}" not in @id={METADATA_FILE}')
        else:
          print(f'**ERROR: @id={METADATA_FILE} does not uniquely exist ')
          success = False
        main_node = [i for i in graph if i["@id"] == "./"][0]

        # iteratively go through graph
        for partI in main_node['hasPart']:
          success = processNode(graph, partI['@id']) and success
        if fileName not in logJson:
          logJson[fileName] = {'params_metadata_json':success}
        else:
          logJson[fileName] = logJson[fileName] | {'params_metadata_json':success}

        # count occurrences of all keys
        counts:dict[str,int] = {}
        for node in graph:
          if node['@id'] in ['./',METADATA_FILE]:
            continue
          for key in node.keys():
            if key in counts:
              counts[key] += 1
            else:
              counts[key] = 1
        view = [ (v,k) for k,v in counts.items() ]
        view.sort(reverse=True)
        if OUTPUT_COUNTS and verbose:
          print('===== Counts (* unspecified)')
          for v,k in view:
            prefix = '   ' if k in KNOWN_KEYS else ' * '
            print(f'{prefix}{k:15}: {v}')
          print('\n\nSuccess:', success)
      return
    ###### BEGIN testELNFile: copy into all export tests in this folder   ######
    # main test function: create stuff, test, ...
    outputFormat = 'print'
    # initialization: create database, destroy on filesystem and database and then create new one
    warnings.filterwarnings('ignore', message='numpy.ufunc size changed')
    warnings.filterwarnings('ignore', message='invalid escape sequence')
    warnings.filterwarnings('ignore', category=ResourceWarning, module='PIL')
    warnings.filterwarnings('ignore', category=ImportWarning)
    projectGroup = 'research'
    self.be = Backend(projectGroup, initConfig=False)

    # change documents
    docIDs = [i.split('|')[-1].strip() for i in self.be.output('procedure',True).split('\n')[2:-1]]
    docIDs +=[i.split('|')[-1].strip() for i in self.be.output('sample',True).split('\n')[2:-1]]
    for idx,docID in enumerate(docIDs):
      doc = self.be.db.getDoc(docID)
      doc['comment'] = f'Test string {idx}'
      self.be.editData(doc)

    # export
    viewProj = self.be.db.getView('viewDocType/x0')
    idProjects  = [i['id'] for i in viewProj]
    self.fileName = str(Path.home()/'temporary_pastaTest.eln')
    status = exportELN(self.be, idProjects, self.fileName, ['procedure','measurement','sample'])
    print(f'Export to: {self.fileName}\n{status}')
    self.assertEqual(status[:21],'Success: exported 17 ','Export unsuccessful')

    # verify eln
    print('\n\nEnd export\n----------------------\nStart verification')
    testELNFile(self.fileName, False)

    # IMPORT NOT IMPLEMENTED YET
    # # remove old
    # docProj = self.be.db.getDoc(idProj)
    # oldPath = self.be.basePath/docProj['-branch'][0]['path']
    # shutil.rmtree(oldPath)
    # allDocs = self.be.db.getView('viewHierarchy/viewHierarchy',    startKey=idProj)
    # for doc in allDocs:
    #   self.be.db.remove(doc['id'])

    # # import
    # print('\n\n---------------\nImport')
    # status = importELN(self.be, self.fileName)
    # print(status)
    # self.assertEqual(status[:7],'Success','Import unsuccessful')

    #test / verify
    print('Number of documents', len(self.be.db.getView('viewHierarchy/viewHierarchy')))
    outputString(outputFormat,'h2','VERIFY DATABASE INTEGRITY')
    outputString(outputFormat,'info', self.be.checkDB(outputStyle='text'))
    outputString(outputFormat,'h2','DONE WITH VERIFY')
    fileCount = 0
    for _, _, files in os.walk(self.be.basePath):
      fileCount+=len(files)
    print(f'Number of files 16 (should be) = {fileCount} (are)')
    self.assertEqual(fileCount, 16, 'Imported entries are not 16, as they should.')
    return

  def tearDown(self):
    logging.info('End Export-import test')
    # Path(self.fileName).unlink()  #remove file
    return


if __name__ == '__main__':
  unittest.main()



""" AttachmentsTableViewModel used for the data hierarchy editor's attachments table view """
#  PASTA-ELN and all its sub-parts are covered by the MIT license.
#
#  Copyright (c) 2023
#
#  Author: Jithu Murugan
#  Filename: attachments_tableview_data_model.py
#
#  You should have received a copy of the license with this file. Please refer the license file for more information.

import logging
from typing import Union

from PySide6.QtWidgets import QWidget

from .tableview_data_model import TableViewModel


class AttachmentsTableViewModel(TableViewModel):
  """
  Abstracted data-model for the data hierarchy editor's attachments table view
  """

  def __init__(self,
               parent: Union[QWidget | None] = None):
    """
    Initialize the data model representing attachments from data hierarchy document in the database
    Args:
      parent (QWidget): Parent view or the widget
    """
    super().__init__(parent)
    self.logger = logging.getLogger(f"{__name__}.{self.__class__.__name__}")
    self.data_set = []
    self.data_name_map = {
      0: "description",
      1: "type",
      2: "delete",
      3: "re-order"
    }
    self.column_widths: dict[int, int] = {
      0: 300,  # Location column width
      1: 600,  # Link column width
      2: 120,  # Delete column width
      3: 120,  # Re-order column width
    }
    self.header_values: list[str] = list(self.data_name_map.values())
    self.columns_count: int = len(self.header_values)



# Form implementation generated from reading ui file 'config_dialog_base.ui'
#
# Created by: PyQt6 UI code generator 6.4.2
#
# WARNING: Any manual changes made to this file will be lost when pyuic6 is
# run again.  Do not edit this file unless you know what you are doing.


from PySide6 import QtCore, QtGui, QtWidgets


class Ui_ConfigDialogBase(object):
  def setupUi(self, ConfigDialogBase):
    ConfigDialogBase.setObjectName("ConfigDialogBase")
    ConfigDialogBase.resize(1070, 192)
    self.gridLayout = QtWidgets.QGridLayout(ConfigDialogBase)
    self.gridLayout.setObjectName("gridLayout")
    self.buttonBox = QtWidgets.QDialogButtonBox(parent=ConfigDialogBase)
    self.buttonBox.setOrientation(QtCore.Qt.Orientation.Horizontal)
    self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.StandardButton.Cancel|QtWidgets.QDialogButtonBox.StandardButton.Save)
    self.buttonBox.setObjectName("buttonBox")
    self.gridLayout.addWidget(self.buttonBox, 2, 0, 1, 1)
    self.mainVerticalLayout = QtWidgets.QVBoxLayout()
    self.mainVerticalLayout.setObjectName("mainVerticalLayout")
    self.serverHorizontalLayout = QtWidgets.QHBoxLayout()
    self.serverHorizontalLayout.setObjectName("serverHorizontalLayout")
    self.dataverseServerLabel = QtWidgets.QLabel(parent=ConfigDialogBase)
    self.dataverseServerLabel.setMinimumSize(QtCore.QSize(200, 0))
    self.dataverseServerLabel.setObjectName("dataverseServerLabel")
    self.serverHorizontalLayout.addWidget(self.dataverseServerLabel)
    self.dataverseServerLineEdit = QtWidgets.QLineEdit(parent=ConfigDialogBase)
    self.dataverseServerLineEdit.setClearButtonEnabled(True)
    self.dataverseServerLineEdit.setObjectName("dataverseServerLineEdit")
    self.serverHorizontalLayout.addWidget(self.dataverseServerLineEdit)
    spacerItem = QtWidgets.QSpacerItem(155, 20, QtWidgets.QSizePolicy.Policy.Preferred, QtWidgets.QSizePolicy.Policy.Minimum)
    self.serverHorizontalLayout.addItem(spacerItem)
    self.mainVerticalLayout.addLayout(self.serverHorizontalLayout)
    spacerItem1 = QtWidgets.QSpacerItem(20, 10, QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Fixed)
    self.mainVerticalLayout.addItem(spacerItem1)
    self.apiTokenHorizontalLayout = QtWidgets.QHBoxLayout()
    self.apiTokenHorizontalLayout.setObjectName("apiTokenHorizontalLayout")
    self.apiTokenLabel = QtWidgets.QLabel(parent=ConfigDialogBase)
    self.apiTokenLabel.setMinimumSize(QtCore.QSize(200, 0))
    self.apiTokenLabel.setObjectName("apiTokenLabel")
    self.apiTokenHorizontalLayout.addWidget(self.apiTokenLabel)
    self.apiTokenLineEdit = QtWidgets.QLineEdit(parent=ConfigDialogBase)
    self.apiTokenLineEdit.setEchoMode(QtWidgets.QLineEdit.EchoMode.Password)
    self.apiTokenLineEdit.setClearButtonEnabled(True)
    self.apiTokenLineEdit.setObjectName("apiTokenLineEdit")
    self.apiTokenHorizontalLayout.addWidget(self.apiTokenLineEdit)
    self.apiTokenHelpPushButton = QtWidgets.QPushButton(parent=ConfigDialogBase)
    self.apiTokenHelpPushButton.setMinimumSize(QtCore.QSize(150, 0))
    self.apiTokenHelpPushButton.setObjectName("apiTokenHelpPushButton")
    self.apiTokenHorizontalLayout.addWidget(self.apiTokenHelpPushButton)
    self.mainVerticalLayout.addLayout(self.apiTokenHorizontalLayout)
    spacerItem2 = QtWidgets.QSpacerItem(20, 10, QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Fixed)
    self.mainVerticalLayout.addItem(spacerItem2)
    self.dataverseListHorizontalLayout = QtWidgets.QHBoxLayout()
    self.dataverseListHorizontalLayout.setObjectName("dataverseListHorizontalLayout")
    self.dataverseListLabel = QtWidgets.QLabel(parent=ConfigDialogBase)
    self.dataverseListLabel.setMinimumSize(QtCore.QSize(200, 0))
    self.dataverseListLabel.setObjectName("dataverseListLabel")
    self.dataverseListHorizontalLayout.addWidget(self.dataverseListLabel)
    self.dataverseListComboBox = QtWidgets.QComboBox(parent=ConfigDialogBase)
    sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Fixed)
    sizePolicy.setHorizontalStretch(0)
    sizePolicy.setVerticalStretch(0)
    sizePolicy.setHeightForWidth(self.dataverseListComboBox.sizePolicy().hasHeightForWidth())
    self.dataverseListComboBox.setSizePolicy(sizePolicy)
    self.dataverseListComboBox.setSizeAdjustPolicy(QtWidgets.QComboBox.SizeAdjustPolicy.AdjustToMinimumContentsLengthWithIcon)
    self.dataverseListComboBox.setObjectName("dataverseListComboBox")
    self.dataverseListHorizontalLayout.addWidget(self.dataverseListComboBox)
    self.dataverseLineEdit = QtWidgets.QLineEdit(parent=ConfigDialogBase)
    self.dataverseLineEdit.setClearButtonEnabled(True)
    self.dataverseLineEdit.setObjectName("dataverseLineEdit")
    self.dataverseListHorizontalLayout.addWidget(self.dataverseLineEdit)
    self.dataverseVerifyLoadPushButton = QtWidgets.QPushButton(parent=ConfigDialogBase)
    self.dataverseVerifyLoadPushButton.setMinimumSize(QtCore.QSize(150, 0))
    self.dataverseVerifyLoadPushButton.setObjectName("dataverseVerifyLoadPushButton")
    self.dataverseListHorizontalLayout.addWidget(self.dataverseVerifyLoadPushButton)
    self.mainVerticalLayout.addLayout(self.dataverseListHorizontalLayout)
    spacerItem3 = QtWidgets.QSpacerItem(20, 10, QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Fixed)
    self.mainVerticalLayout.addItem(spacerItem3)
    self.gridLayout.addLayout(self.mainVerticalLayout, 0, 0, 1, 1)

    self.retranslateUi(ConfigDialogBase)
    self.buttonBox.accepted.connect(ConfigDialogBase.accept) # type: ignore
    self.buttonBox.rejected.connect(ConfigDialogBase.reject) # type: ignore
    QtCore.QMetaObject.connectSlotsByName(ConfigDialogBase)

  def retranslateUi(self, ConfigDialogBase):
    _translate = QtCore.QCoreApplication.translate
    ConfigDialogBase.setWindowTitle(_translate("ConfigDialogBase", "Dataverse configuration"))
    self.dataverseServerLabel.setText(_translate("ConfigDialogBase", "Dataverse URL"))
    self.dataverseServerLineEdit.setToolTip(_translate("ConfigDialogBase", "Enter the dataverse server URL."))
    self.dataverseServerLineEdit.setPlaceholderText(_translate("ConfigDialogBase", "Enter the dataverse server URL, e.g. https://data.fz-juelich.de/"))
    self.apiTokenLabel.setText(_translate("ConfigDialogBase", "API token"))
    self.apiTokenLineEdit.setToolTip(_translate("ConfigDialogBase", "Enter the API token."))
    self.apiTokenLineEdit.setPlaceholderText(_translate("ConfigDialogBase", "Enter the API token, e.g. c6527048-5bdc-48b0-a1d5-ed1b62c4513b"))
    self.apiTokenHelpPushButton.setToolTip(_translate("ConfigDialogBase", "Navigate to the help page for generating dataverse API token."))
    self.apiTokenHelpPushButton.setText(_translate("ConfigDialogBase", "Help"))
    self.dataverseListLabel.setText(_translate("ConfigDialogBase", "Dataverse list"))
    self.dataverseListComboBox.setToolTip(_translate("ConfigDialogBase", "Displays the dataverse list from the server."))
    self.dataverseLineEdit.setToolTip(_translate("ConfigDialogBase", "Displays the ID of the selected dataverse from the list."))
    self.dataverseLineEdit.setPlaceholderText(_translate("ConfigDialogBase", "Selected Dataverse ID"))
    self.dataverseVerifyLoadPushButton.setToolTip(_translate("ConfigDialogBase", "Verify API credentials and load available dataverse list from the server."))
    self.dataverseVerifyLoadPushButton.setText(_translate("ConfigDialogBase", "Verify && Load"))


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    ConfigDialogBase = QtWidgets.QDialog()
    ui = Ui_ConfigDialogBase()
    ui.setupUi(ConfigDialogBase)
    ConfigDialogBase.show()
    sys.exit(app.exec())



""" Widget: setup tab inside the configuration dialog window """
import ctypes
import logging
import os
from enum import Enum
from pathlib import Path
from typing import Any, Callable

from PySide6.QtWidgets import QFileDialog, QMessageBox, QProgressBar, QTextEdit, QVBoxLayout, \
  QWidget  # pylint: disable=no-name-in-module

from ..dataverse.database_api import DatabaseAPI
from ..fixedStringsJson import couchDBWindows, exampleDataWindows, restartPastaWindows, setupTextWindows
from ..guiCommunicate import Communicate
from ..guiStyle import TextButton, widgetAndLayout
from ..installationTools import checkForDotNetVersion, configuration, couchdb, createShortcut, dataHierarchy, \
  exampleData
from ..miscTools import restart


class ConfigurationSetup(QWidget):
  """
  Main class
  """
  def __init__(self, comm:Communicate, callbackFinished:Callable[[],None]):
    """
    Initialization

    Args:
      comm (Communicate): communication
      callbackFinished (function): callback function to call upon end
    """
    super().__init__()
    self.mainL = QVBoxLayout()
    self.setMinimumWidth(400)
    self.setMinimumHeight(500)
    self.setLayout(self.mainL)
    self.callbackFinished = callbackFinished
    self.comm = comm

    #widget 1 = screen 1
    self.screen1W, screen1L = widgetAndLayout('V', self.mainL)
    self.text1 = QTextEdit()
    self.mainText = setupTextWindows
    self.text1.setMarkdown(self.mainText)
    screen1L.addWidget(self.text1)
    self.progress1 = QProgressBar(self.screen1W)
    self.progress1.setMaximum(24)
    self.progress1.hide()
    screen1L.addWidget(self.progress1)

    _, footerL = widgetAndLayout('H', screen1L, 's')
    self.button1 = TextButton('Start analyse and repair', self, [Command.ANALYSE], footerL)
    self.button2 = TextButton('Finished',                 self, [Command.FINISHED], footerL)
    self.button2.hide()


  def callbackProgress(self, number:int) -> None:
    """
    Increase progressbar by moving to number

    Args:
      number (int): integer to move to
    """
    self.progress1.setValue(number)
    return


  def execute(self, command:list[Any]) -> None:
    """
    Main method that does all the analysis: open dialogs, ...
    """
    if command[0] is Command.ANALYSE:
      flagContinue = True
      flagInstalledSoftware = False
      logging.info('Windows setup analyse start')

      #Couchdb
      if flagContinue:
        res = couchdb('test')
        if res =='':
          self.mainText = self.mainText.replace('- CouchDB','- CouchDB is installed' )
          self.text1.setMarkdown(self.mainText)
        else:
          button = QMessageBox.question(self, "CouchDB installation", couchDBWindows)
          if button == QMessageBox.StandardButton.Yes:
            try:
              isAdmin = os.getuid() == 0
            except AttributeError:
              isAdmin = ctypes.windll.shell32.IsUserAnAdmin() == 1 # type: ignore[attr-defined]
            if not isAdmin:
              QMessageBox.information(self,'Administrator rights required', \
                          'You require administrator rights for your user. I exit installation now.')
              self.mainText = self.mainText.replace('- CouchDB','- CouchDB: no admin rights' )
              self.text1.setMarkdown(self.mainText)
              flagContinue = False
            elif not checkForDotNetVersion():
              QMessageBox.information(self,'.NET version 3.5 required', \
                          '.NET version 3.5 is required for the installation of couchDB. I exit installation now.')
              self.mainText = self.mainText.replace('- CouchDB','- CouchDB: no .NET 3.5' )
              self.text1.setMarkdown(self.mainText)
              flagContinue = False
            else:
              res = couchdb('install')
              flagInstalledSoftware = True
              if len(res.split('|'))==3:
                password=res.split('|')[1]
              else:
                logging.error('Could not retrieve password :%s',str(res))
          else:
            self.mainText = self.mainText.replace('- CouchDB','- CouchDB: user chose to NOT install' )
            self.text1.setMarkdown(self.mainText)
            flagContinue = False

      #Configuration
      if flagContinue:
        res = configuration('test')
        if res =='':
          self.mainText = self.mainText.replace('- Configuration of preferences','- Configuration of preferences is acceptable' )
          self.text1.setMarkdown(self.mainText)
        else:
          button = QMessageBox.question(self, "PASTA-ELN configuration", "Do you want to create/repain the configuration.")
          if button == QMessageBox.StandardButton.Yes:
            dirName = QFileDialog.getExistingDirectory(self,'Create and select directory for scientific data',str(Path.home()/'Documents'))
            configuration('repair','admin', password, Path(dirName))
            flagInstalledSoftware = True
          else:
            self.mainText = self.mainText.replace('- Configuration of preferences','- Configuration: user chose to NOT install' )
            self.text1.setMarkdown(self.mainText)
            flagContinue = False

      #DataHierarchy
      if flagContinue:
        res = dataHierarchy('test')
        if '**ERROR' not in res:
          self.mainText = self.mainText.replace('- DataHierarchy of the datastructure','- DataHierarchy of the datastructure is acceptable\n'+res )
          self.text1.setMarkdown(self.mainText)
        else:
          # button = QMessageBox.question(self, "PASTA-ELN dataHierarchy", "Do you want to create the default dataHierarchy?")
          # if button == QMessageBox.Yes:
          dataHierarchy('install')
          # else:
          #   self.mainText = self.mainText.replace('- DataHierarchy of the datastructure','- DataHierarchy: user chose to NOT install' )
          #   self.text1.setMarkdown(self.mainText)
          #   flagContinue = False

      #Shortcut
      if flagContinue:
        button = QMessageBox.question(self, "Create shortcut", "Do you want to create the shortcut for PASTA-ELN on desktop?")
        if button == QMessageBox.StandardButton.Yes:
          createShortcut()
          self.mainText = self.mainText.replace('- Shortcut creation', '- User selected to add a shortcut' )
        else:
          self.mainText = self.mainText.replace('- Shortcut creation', '- User selected to NOT add a shortcut' )
        self.text1.setMarkdown(self.mainText)

      #If installed, restart
      if flagInstalledSoftware:
        QMessageBox.information(self,'PASTA-ELN restart required', restartPastaWindows)
        restart()

      #Example data
      if flagContinue:
        button = QMessageBox.question(self, "Example data", exampleDataWindows)
        if button == QMessageBox.StandardButton.Yes:
          self.progress1.show()
          if (self.comm.backend.basePath/'pastasExampleProject').exists():
            button1 = QMessageBox.question(self, "Example data", 'Data exists. Should I reset?')
            if button1 == QMessageBox.StandardButton.Yes:
              exampleData(True, self.callbackProgress)
            else:
              self.mainText = self.mainText.replace('- Example data', '- Example data exists and should not be deleted.')
          else:
            exampleData(False, self.callbackProgress)
            self.mainText = self.mainText.replace('- Example data', '- Example data was added')
        else:
          self.mainText = self.mainText.replace('- Example data', '- Example data was NOT added, per user choice')
          flagContinue = False
        self.text1.setMarkdown(self.mainText)

      #at end
      self.button1.hide()
      self.button2.show()
      logging.info('Windows setup analyse end')
    elif command[0] is Command.FINISHED: # What do do when setup is finished: success or unsuccessfully
      db_api = DatabaseAPI()
      db_api.initialize_database()
      restart()
      # self.callbackFinished()
    return



class Command(Enum):
  """ Commands used in this file """
  ANALYSE   = 1
  FINISHED  = 2
